@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,13 +17,12 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
+#include "dbupgrade.h"
 
+#include "common.h"
 #include "db.h"
 #include "log.h"
-#include "sysinfo.h"
 #include "zbxdbupgrade.h"
-#include "dbupgrade.h"
 
 typedef struct
 {
@@ -40,65 +39,46 @@ zbx_db_version_t;
 #	define ZBX_DROP_FK		" drop constraint"
 #endif
 
-#if defined(HAVE_IBM_DB2)
-#	define ZBX_DB_ALTER_COLUMN	" alter column"
-#elif defined(HAVE_POSTGRESQL)
+#if defined(HAVE_POSTGRESQL)
 #	define ZBX_DB_ALTER_COLUMN	" alter"
 #else
 #	define ZBX_DB_ALTER_COLUMN	" modify"
 #endif
 
-#if defined(HAVE_IBM_DB2)
-#	define ZBX_DB_SET_TYPE		" set data type"
-#elif defined(HAVE_POSTGRESQL)
+#if defined(HAVE_POSTGRESQL)
 #	define ZBX_DB_SET_TYPE		" type"
 #else
 #	define ZBX_DB_SET_TYPE		""
 #endif
 
-#if defined(HAVE_IBM_DB2) || defined(HAVE_POSTGRESQL)
-#	define ZBX_TYPE_ID_STR		"bigint"
-#elif defined(HAVE_MYSQL)
+/* NOTE: Do not forget to sync changes in ZBX_TYPE_*_STR defines for Oracle with zbx_oracle_column_type()! */
+
+#if defined(HAVE_MYSQL)
 #	define ZBX_TYPE_ID_STR		"bigint unsigned"
+#	define ZBX_TYPE_FLOAT_STR	"double precision"
+#	define ZBX_TYPE_UINT_STR	"bigint unsigned"
+#	define ZBX_TYPE_LONGTEXT_STR	"longtext"
 #elif defined(HAVE_ORACLE)
 #	define ZBX_TYPE_ID_STR		"number(20)"
+#	define ZBX_TYPE_FLOAT_STR	"binary_double"
+#	define ZBX_TYPE_UINT_STR	"number(20)"
+#	define ZBX_TYPE_LONGTEXT_STR	"nclob"
+#elif defined(HAVE_POSTGRESQL)
+#	define ZBX_TYPE_ID_STR		"bigint"
+#	define ZBX_TYPE_FLOAT_STR	"double precision"
+#	define ZBX_TYPE_UINT_STR	"numeric(20)"
+#	define ZBX_TYPE_LONGTEXT_STR	"text"
 #endif
 
-#ifdef HAVE_ORACLE
+#if defined(HAVE_ORACLE)
 #	define ZBX_TYPE_INT_STR		"number(10)"
 #	define ZBX_TYPE_CHAR_STR	"nvarchar2"
+#	define ZBX_TYPE_SHORTTEXT_STR	"nvarchar2(2048)"
+#	define ZBX_TYPE_TEXT_STR	"nclob"
 #else
 #	define ZBX_TYPE_INT_STR		"integer"
 #	define ZBX_TYPE_CHAR_STR	"varchar"
-#endif
-
-#if defined(HAVE_IBM_DB2)
-#	define ZBX_TYPE_FLOAT_STR	"decfloat(16)"
-#	define ZBX_TYPE_UINT_STR	"bigint"
-#elif defined(HAVE_MYSQL)
-#	define ZBX_TYPE_FLOAT_STR	"double(16,4)"
-#	define ZBX_TYPE_UINT_STR	"bigint unsigned"
-#elif defined(HAVE_ORACLE)
-#	define ZBX_TYPE_FLOAT_STR	"number(20,4)"
-#	define ZBX_TYPE_UINT_STR	"number(20)"
-#elif defined(HAVE_POSTGRESQL)
-#	define ZBX_TYPE_FLOAT_STR	"numeric(16,4)"
-#	define ZBX_TYPE_UINT_STR	"numeric(20)"
-#endif
-
-#if defined(HAVE_IBM_DB2)
-#	define ZBX_TYPE_SHORTTEXT_STR	"varchar(2048)"
-#elif defined(HAVE_ORACLE)
-#	define ZBX_TYPE_SHORTTEXT_STR	"nvarchar2(2048)"
-#else
 #	define ZBX_TYPE_SHORTTEXT_STR	"text"
-#endif
-
-#if defined(HAVE_IBM_DB2)
-#	define ZBX_TYPE_TEXT_STR	"varchar(2048)"
-#elif defined(HAVE_ORACLE)
-#	define ZBX_TYPE_TEXT_STR	"nclob"
-#else
 #	define ZBX_TYPE_TEXT_STR	"text"
 #endif
 
@@ -106,7 +86,6 @@ zbx_db_version_t;
 
 extern unsigned char	program_type;
 
-
 #ifndef HAVE_SQLITE3
 static void	DBfield_type_string(char **sql, size_t *sql_alloc, size_t *sql_offset, const ZBX_FIELD *field)
 {
@@ -127,20 +106,72 @@ static void	DBfield_type_string(char **sql, size_t *sql_alloc, size_t *sql_offse
 		case ZBX_TYPE_UINT:
 			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_TYPE_UINT_STR);
 			break;
+		case ZBX_TYPE_LONGTEXT:
+			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_TYPE_LONGTEXT_STR);
+			break;
 		case ZBX_TYPE_SHORTTEXT:
 			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_TYPE_SHORTTEXT_STR);
 			break;
 		case ZBX_TYPE_TEXT:
 			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_TYPE_TEXT_STR);
 			break;
+		case ZBX_TYPE_CUID:
+			zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s(%d)", ZBX_TYPE_CHAR_STR, CUID_LEN - 1);
+			break;
 		default:
 			assert(0);
 	}
 }
 
+#ifdef HAVE_ORACLE
+typedef enum
+{
+	ZBX_ORACLE_COLUMN_TYPE_NUMERIC,
+	ZBX_ORACLE_COLUMN_TYPE_CHARACTER,
+	ZBX_ORACLE_COLUMN_TYPE_DOUBLE,
+	ZBX_ORACLE_COLUMN_TYPE_UNKNOWN
+}
+zbx_oracle_column_type_t;
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: determine whether column type is character or numeric             *
+ *                                                                            *
+ * Parameters: field_type - [IN] column type in Zabbix definitions            *
+ *                                                                            *
+ * Return value: column type (character/raw, numeric) in Oracle definitions   *
+ *                                                                            *
+ * Comments: The size of a character or raw column or the precision of a      *
+ *           numeric column can be changed, whether or not all the rows       *
+ *           contain nulls. Otherwise in order to change the datatype of a    *
+ *           column all rows of the column must contain nulls.                *
+ *                                                                            *
+ ******************************************************************************/
+static zbx_oracle_column_type_t	zbx_oracle_column_type(unsigned char field_type)
+{
+	switch (field_type)
+	{
+		case ZBX_TYPE_ID:
+		case ZBX_TYPE_INT:
+		case ZBX_TYPE_UINT:
+			return ZBX_ORACLE_COLUMN_TYPE_NUMERIC;
+		case ZBX_TYPE_CHAR:
+		case ZBX_TYPE_LONGTEXT:
+		case ZBX_TYPE_SHORTTEXT:
+		case ZBX_TYPE_TEXT:
+			return ZBX_ORACLE_COLUMN_TYPE_CHARACTER;
+		case ZBX_TYPE_FLOAT:
+			return ZBX_ORACLE_COLUMN_TYPE_DOUBLE;
+		default:
+			THIS_SHOULD_NEVER_HAPPEN;
+			return ZBX_ORACLE_COLUMN_TYPE_UNKNOWN;
+	}
+}
+#endif
+
 static void	DBfield_definition_string(char **sql, size_t *sql_alloc, size_t *sql_offset, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s ", field->name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, ZBX_FS_SQL_NAME " ", field->name);
 	DBfield_type_string(sql, sql_alloc, sql_offset, field);
 	if (NULL != field->default_value)
 	{
@@ -204,21 +235,64 @@ static void	DBcreate_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "\n)" ZBX_DB_TABLE_OPTIONS);
 }
 
+static void	DBrename_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *table_name,
+		const char *new_name)
+{
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " rename to " ZBX_FS_SQL_NAME,
+			table_name, new_name);
+}
+
 static void	DBdrop_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *table_name)
 {
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "drop table %s", table_name);
 }
 
-static void	DBmodify_field_type_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
+static void	DBset_default_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const ZBX_FIELD *field)
 {
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s" ZBX_DB_ALTER_COLUMN " ", table_name);
 
+#if defined(HAVE_MYSQL)
+	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
+#elif defined(HAVE_ORACLE)
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s default '%s'", field->name, field->default_value);
+#else
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s set default '%s'", field->name, field->default_value);
+#endif
+}
+
+static void	DBdrop_default_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
+		const char *table_name, const ZBX_FIELD *field)
+{
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s" ZBX_DB_ALTER_COLUMN " ", table_name);
+
+#if defined(HAVE_MYSQL)
+	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
+#elif defined(HAVE_ORACLE)
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s default null", field->name);
+#else
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s drop default", field->name);
+#endif
+}
+
+static void	DBmodify_field_type_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
+		const char *table_name, const ZBX_FIELD *field)
+{
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME ZBX_DB_ALTER_COLUMN " ",
+			table_name);
+
 #ifdef HAVE_MYSQL
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
 #else
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s" ZBX_DB_SET_TYPE " ", field->name);
 	DBfield_type_string(sql, sql_alloc, sql_offset, field);
+#ifdef HAVE_POSTGRESQL
+	if (NULL != field->default_value)
+	{
+		zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ";\n");
+		DBset_default_sql(sql, sql_alloc, sql_offset, table_name, field);
+	}
+#endif
 #endif
 }
 
@@ -250,37 +324,24 @@ static void	DBset_not_null_sql(char **sql, size_t *sql_alloc, size_t *sql_offset
 #endif
 }
 
-static void	DBset_default_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
-		const char *table_name, const ZBX_FIELD *field)
-{
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s" ZBX_DB_ALTER_COLUMN " ", table_name);
-
-#if defined(HAVE_MYSQL)
-	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
-#elif defined(HAVE_ORACLE)
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s default '%s'", field->name, field->default_value);
-#else
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s set default '%s'", field->name, field->default_value);
-#endif
-}
-
 static void	DBadd_field_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s add ", table_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " add ", table_name);
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
 }
 
 static void	DBrename_field_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const char *field_name, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s ", table_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " ", table_name);
 
 #ifdef HAVE_MYSQL
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "change column %s ", field_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "change column " ZBX_FS_SQL_NAME " ", field_name);
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
 #else
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename column %s to %s", field_name, field->name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename column " ZBX_FS_SQL_NAME " to " ZBX_FS_SQL_NAME,
+			field_name, field->name);
 #endif
 }
 
@@ -305,20 +366,23 @@ static void	DBdrop_index_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "drop index %s", index_name);
 #ifdef HAVE_MYSQL
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, " on %s", table_name);
+#else
+	ZBX_UNUSED(table_name);
 #endif
 }
 
 static void	DBrename_index_sql(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *table_name,
 		const char *old_name, const char *new_name, const char *fields, int unique)
 {
-#if defined(HAVE_IBM_DB2)
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename index %s to %s", old_name, new_name);
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	DBcreate_index_sql(sql, sql_alloc, sql_offset, table_name, new_name, fields, unique);
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ";\n");
 	DBdrop_index_sql(sql, sql_alloc, sql_offset, table_name, old_name);
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ";\n");
 #elif defined(HAVE_ORACLE) || defined(HAVE_POSTGRESQL)
+	ZBX_UNUSED(table_name);
+	ZBX_UNUSED(fields);
+	ZBX_UNUSED(unique);
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter index %s rename to %s", old_name, new_name);
 #endif
 }
@@ -327,8 +391,9 @@ static void	DBadd_foreign_key_sql(char **sql, size_t *sql_alloc, size_t *sql_off
 		const char *table_name, int id, const ZBX_FIELD *field)
 {
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset,
-			"alter table %s add constraint c_%s_%d foreign key (%s) references %s (%s)",
-			table_name, table_name, id, field->name, field->fk_table, field->fk_field);
+			"alter table " ZBX_FS_SQL_NAME " add constraint c_%s_%d foreign key (" ZBX_FS_SQL_NAME ")"
+					" references " ZBX_FS_SQL_NAME " (" ZBX_FS_SQL_NAME ")", table_name, table_name,
+					id, field->name, field->fk_table, field->fk_field);
 	if (0 != (field->fk_flags & ZBX_FK_CASCADE_DELETE))
 		zbx_strcpy_alloc(sql, sql_alloc, sql_offset, " on delete cascade");
 }
@@ -340,18 +405,6 @@ static void	DBdrop_foreign_key_sql(char **sql, size_t *sql_alloc, size_t *sql_of
 			table_name, table_name, id);
 }
 
-static int	DBreorg_table(const char *table_name)
-{
-#ifdef HAVE_IBM_DB2
-	if (ZBX_DB_OK <= DBexecute("call sysproc.admin_cmd ('reorg table %s')", table_name))
-		return SUCCEED;
-
-	return FAIL;
-#else
-	return SUCCEED;
-#endif
-}
-
 int	DBcreate_table(const ZBX_TABLE *table)
 {
 	char	*sql = NULL;
@@ -368,6 +421,22 @@ int	DBcreate_table(const ZBX_TABLE *table)
 	return ret;
 }
 
+int	DBrename_table(const char *table_name, const char *new_name)
+{
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+
+	DBrename_table_sql(&sql, &sql_alloc, &sql_offset, table_name, new_name);
+
+	if (ZBX_DB_OK <= DBexecute("%s", sql))
+		ret = SUCCEED;
+
+	zbx_free(sql);
+
+	return ret;
+}
+
 int	DBdrop_table(const char *table_name)
 {
 	char	*sql = NULL;
@@ -393,7 +462,7 @@ int	DBadd_field(const char *table_name, const ZBX_FIELD *field)
 	DBadd_field_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -409,23 +478,75 @@ int	DBrename_field(const char *table_name, const char *field_name, const ZBX_FIE
 	DBrename_field_sql(&sql, &sql_alloc, &sql_offset, table_name, field_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
 	return ret;
 }
 
-int	DBmodify_field_type(const char *table_name, const ZBX_FIELD *field)
+#ifdef HAVE_ORACLE
+static int	DBmodify_field_type_with_copy(const char *table_name, const ZBX_FIELD *field)
+{
+#define ZBX_OLD_FIELD	"zbx_old_tmp"
+
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, "alter table %s rename column %s to " ZBX_OLD_FIELD,
+			table_name, field->name);
+
+	if (ZBX_DB_OK > DBexecute("%s", sql))
+		goto out;
+
+	if (ZBX_DB_OK > DBadd_field(table_name, field))
+		goto out;
+
+	sql_offset = 0;
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, "update %s set %s=" ZBX_OLD_FIELD, table_name,
+			field->name);
+
+	if (ZBX_DB_OK > DBexecute("%s", sql))
+		goto out;
+
+	ret = DBdrop_field(table_name, ZBX_OLD_FIELD);
+out:
+	zbx_free(sql);
+
+	return ret;
+
+#undef ZBX_OLD_FIELD
+}
+#endif
+
+int	DBmodify_field_type(const char *table_name, const ZBX_FIELD *field, const ZBX_FIELD *old_field)
 {
 	char	*sql = NULL;
 	size_t	sql_alloc = 0, sql_offset = 0;
 	int	ret = FAIL;
 
+#ifndef HAVE_ORACLE
+	ZBX_UNUSED(old_field);
+#else
+	/* Oracle cannot change column type in a general case if column contents are not null. Conversions like   */
+	/* number -> nvarchar2 or nvarchar2 -> nclob need special processing. New column is created with desired  */
+	/* datatype and data from old column is copied there. Then old column is dropped. This method does not    */
+	/* preserve column order.                                                                                 */
+	/* NOTE: Existing column indexes and constraints are not respected by the current implementation!         */
+
+	if (NULL != old_field && (zbx_oracle_column_type(old_field->type) != zbx_oracle_column_type(field->type) ||
+			ZBX_ORACLE_COLUMN_TYPE_DOUBLE == zbx_oracle_column_type(field->type) ||
+			((ZBX_TYPE_TEXT == field->type || ZBX_TYPE_LONGTEXT == field->type) &&
+				(ZBX_TYPE_SHORTTEXT == old_field->type || ZBX_TYPE_CHAR == old_field->type))))
+	{
+		return DBmodify_field_type_with_copy(table_name, field);
+	}
+#endif
 	DBmodify_field_type_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -441,7 +562,7 @@ int	DBset_not_null(const char *table_name, const ZBX_FIELD *field)
 	DBset_not_null_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -457,7 +578,23 @@ int	DBset_default(const char *table_name, const ZBX_FIELD *field)
 	DBset_default_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
+
+	zbx_free(sql);
+
+	return ret;
+}
+
+int	DBdrop_default(const char *table_name, const ZBX_FIELD *field)
+{
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+
+	DBdrop_default_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
+
+	if (ZBX_DB_OK <= DBexecute("%s", sql))
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -473,7 +610,7 @@ int	DBdrop_not_null(const char *table_name, const ZBX_FIELD *field)
 	DBdrop_not_null_sql(&sql, &sql_alloc, &sql_offset, table_name, field);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -489,7 +626,7 @@ int	DBdrop_field(const char *table_name, const char *field_name)
 	DBdrop_field_sql(&sql, &sql_alloc, &sql_offset, table_name, field_name);
 
 	if (ZBX_DB_OK <= DBexecute("%s", sql))
-		ret = DBreorg_table(table_name);
+		ret = SUCCEED;
 
 	zbx_free(sql);
 
@@ -585,7 +722,8 @@ static int	DBcreate_dbversion_table(void)
 					{"mandatory", "0", NULL, NULL, 0, ZBX_TYPE_INT, ZBX_NOTNULL, 0},
 					{"optional", "0", NULL, NULL, 0, ZBX_TYPE_INT, ZBX_NOTNULL, 0},
 					{NULL}
-				}
+				},
+				NULL
 			};
 	int		ret;
 
@@ -598,9 +736,8 @@ static int	DBcreate_dbversion_table(void)
 			ret = FAIL;
 		}
 	}
-	DBend(ret);
 
-	return ret;
+	return DBend(ret);
 }
 
 static int	DBset_version(int version, unsigned char mandatory)
@@ -627,6 +764,24 @@ extern zbx_dbpatch_t	DBPATCH_VERSION(2030)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(2040)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(2050)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(3000)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3010)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3020)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3030)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3040)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3050)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4000)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4010)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4020)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4030)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4040)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4050)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5000)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5010)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5020)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5030)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5040)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(5050)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(6000)[];
 
 static zbx_db_version_t dbversions[] = {
 	{DBPATCH_VERSION(2010), "2.2 development"},
@@ -635,6 +790,24 @@ static zbx_db_version_t dbversions[] = {
 	{DBPATCH_VERSION(2040), "2.4 maintenance"},
 	{DBPATCH_VERSION(2050), "3.0 development"},
 	{DBPATCH_VERSION(3000), "3.0 maintenance"},
+	{DBPATCH_VERSION(3010), "3.2 development"},
+	{DBPATCH_VERSION(3020), "3.2 maintenance"},
+	{DBPATCH_VERSION(3030), "3.4 development"},
+	{DBPATCH_VERSION(3040), "3.4 maintenance"},
+	{DBPATCH_VERSION(3050), "4.0 development"},
+	{DBPATCH_VERSION(4000), "4.0 maintenance"},
+	{DBPATCH_VERSION(4010), "4.2 development"},
+	{DBPATCH_VERSION(4020), "4.2 maintenance"},
+	{DBPATCH_VERSION(4030), "4.4 development"},
+	{DBPATCH_VERSION(4040), "4.4 maintenance"},
+	{DBPATCH_VERSION(4050), "5.0 development"},
+	{DBPATCH_VERSION(5000), "5.0 maintenance"},
+	{DBPATCH_VERSION(5010), "5.2 development"},
+	{DBPATCH_VERSION(5020), "5.2 maintenance"},
+	{DBPATCH_VERSION(5030), "5.4 development"},
+	{DBPATCH_VERSION(5040), "5.4 maintenance"},
+	{DBPATCH_VERSION(5050), "6.0 development"},
+	{DBPATCH_VERSION(6000), "6.0 maintenance"},
 	{NULL}
 };
 
@@ -664,7 +837,7 @@ static void	DBget_version(int *mandatory, int *optional)
 
 int	DBcheck_version(void)
 {
-	const char		*__function_name = "DBcheck_version";
+#define ZBX_DB_WAIT_UPGRADE	10
 	const char		*dbversion_table_name = "dbversion";
 	int			db_mandatory, db_optional, required, ret = FAIL, i;
 	zbx_db_version_t	*dbversion;
@@ -673,7 +846,7 @@ int	DBcheck_version(void)
 #ifndef HAVE_SQLITE3
 	int			total = 0, current = 0, completed, last_completed = -1, optional_num = 0;
 #endif
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	required = ZBX_FIRST_DB_VERSION;
 
@@ -695,8 +868,7 @@ int	DBcheck_version(void)
 	if (SUCCEED != DBtable_exists(dbversion_table_name))
 	{
 #ifndef HAVE_SQLITE3
-		zabbix_log(LOG_LEVEL_DEBUG, "%s() \"%s\" does not exist",
-				__function_name, dbversion_table_name);
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() \"%s\" does not exist", __func__, dbversion_table_name);
 
 		if (SUCCEED != DBfield_exists("config", "server_check_interval"))
 		{
@@ -768,23 +940,54 @@ int	DBcheck_version(void)
 
 	for (dbversion = dbversions; NULL != dbversion->patches; dbversion++)
 	{
-		zbx_dbpatch_t	*patches = dbversion->patches;
+		patches = dbversion->patches;
 
 		for (i = 0; 0 != patches[i].version; i++)
 		{
+			static sigset_t	orig_mask, mask;
+			DB_RESULT	result;
+			DB_ROW		row;
+
 			if (db_optional >= patches[i].version)
 				continue;
 
+			/* block signals to prevent interruption of statements that cause an implicit commit */
+			sigemptyset(&mask);
+			sigaddset(&mask, SIGTERM);
+			sigaddset(&mask, SIGINT);
+			sigaddset(&mask, SIGQUIT);
+
+			if (0 > sigprocmask(SIG_BLOCK, &mask, &orig_mask))
+				zabbix_log(LOG_LEVEL_WARNING, "cannot set sigprocmask to block the user signal");
+
 			DBbegin();
 
-			/* skipping the duplicated patches */
-			if ((0 != patches[i].duplicates && patches[i].duplicates <= db_optional) ||
-					SUCCEED == (ret = patches[i].function()))
+			result = DBselect("select optional,mandatory from dbversion" ZBX_FOR_UPDATE);
+			if (NULL != (row = DBfetch(result)))
+				db_optional = atoi(row[0]);
+
+			DBfree_result(result);
+			if (db_optional >= patches[i].version)
 			{
-				ret = DBset_version(patches[i].version, patches[i].mandatory);
+				zabbix_log(LOG_LEVEL_INFORMATION, "cannot perform database upgrade:"
+						" patch with version %08d was already performed by other node",
+						patches[i].version);
+				ret = FAIL;
+			}
+			else
+			{
+				/* skipping the duplicated patches */
+				if ((0 != patches[i].duplicates && patches[i].duplicates <= db_optional) ||
+						SUCCEED == (ret = patches[i].function()))
+				{
+					ret = DBset_version(patches[i].version, patches[i].mandatory);
+				}
 			}
 
-			DBend(ret);
+			ret = DBend(ret);
+
+			if (0 > sigprocmask(SIG_SETMASK, &orig_mask, NULL))
+				zabbix_log(LOG_LEVEL_WARNING,"cannot restore sigprocmask");
 
 			if (SUCCEED != ret)
 				break;
@@ -806,13 +1009,192 @@ int	DBcheck_version(void)
 	if (SUCCEED == ret)
 		zabbix_log(LOG_LEVEL_WARNING, "database upgrade fully completed");
 	else
-		zabbix_log(LOG_LEVEL_CRIT, "database upgrade failed");
+	{
+		zabbix_log(LOG_LEVEL_CRIT, "database upgrade failed on patch %08d, exiting in %d seconds",
+				patches[i].version, ZBX_DB_WAIT_UPGRADE);
+		sleep(ZBX_DB_WAIT_UPGRADE);
+	}
 #endif	/* not HAVE_SQLITE3 */
 
 out:
 	DBclose();
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
+
+	return ret;
+#undef ZBX_DB_WAIT_UPGRADE
+}
+
+int	DBcheck_double_type(void)
+{
+	DB_RESULT	result;
+	DB_ROW		row;
+	char		*sql = NULL;
+	const int	total_dbl_cols = 4;
+	int		ret = FAIL;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
+
+	DBconnect(ZBX_DB_CONNECT_NORMAL);
+
+#if defined(HAVE_MYSQL)
+	sql = DBdyn_escape_string(CONFIG_DBNAME);
+	sql = zbx_dsprintf(sql, "select count(*) from information_schema.columns"
+			" where table_schema='%s' and column_type='double'", sql);
+#elif defined(HAVE_POSTGRESQL)
+	sql = DBdyn_escape_string(NULL == CONFIG_DBSCHEMA || '\0' == *CONFIG_DBSCHEMA ? "public" : CONFIG_DBSCHEMA);
+	sql = zbx_dsprintf(sql, "select count(*) from information_schema.columns"
+			" where table_schema='%s' and data_type='double precision'", sql);
+#elif defined(HAVE_ORACLE)
+	sql = zbx_strdup(sql, "select count(*) from user_tab_columns"
+			" where data_type='BINARY_DOUBLE'");
+#elif defined(HAVE_SQLITE3)
+	/* upgrade patch is not required for sqlite3 */
+	ret = SUCCEED;
+	goto out;
+#endif
+
+	if (NULL == (result = DBselect("%s"
+			" and ((lower(table_name)='trends'"
+					" and (lower(column_name) in ('value_min', 'value_avg', 'value_max')))"
+			" or (lower(table_name)='history' and lower(column_name)='value'))", sql)))
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "cannot select records with columns information");
+		goto out;
+	}
+
+	if (NULL != (row = DBfetch(result)) && total_dbl_cols == atoi(row[0]))
+		ret = SUCCEED;
+
+	DBfree_result(result);
+out:
+	DBclose();
+	zbx_free(sql);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
 	return ret;
 }
+#ifndef HAVE_SQLITE3
+int	zbx_dbupgrade_attach_trigger_with_function_on_insert(const char *table_name,
+		const char *original_column_name, const char *indexed_column_name, const char *function,
+		const char *idname)
+{
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+#ifdef HAVE_ORACLE
+	ZBX_UNUSED(idname);
+
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create trigger %s_%s_insert\n"
+			"before insert on %s for each row\n"
+			"begin\n"
+				":new.%s:=%s(:new.%s);\n"
+			"end;",
+			table_name, indexed_column_name, table_name, indexed_column_name, function,
+			original_column_name);
+#elif HAVE_MYSQL
+	ZBX_UNUSED(idname);
+
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create trigger %s_%s_insert\n"
+			"before insert on %s for each row\n"
+				"set new.%s=%s(new.%s)",
+			table_name, indexed_column_name, table_name, indexed_column_name, function,
+			original_column_name);
+#elif defined(HAVE_POSTGRESQL)
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create or replace function %s_%s_%s()\n"
+			"returns trigger language plpgsql AS $func$\n"
+			"begin\n"
+				"update %s set %s=%s(%s)\n"
+				"where %s=new.%s;\n"
+				"return null;\n"
+			"end $func$;\n"
+
+			"create trigger %s_%s_insert after insert\n"
+				"on %s\n"
+				"for each row execute function %s_%s_%s();",
+			table_name, indexed_column_name, function, table_name, indexed_column_name, function,
+			original_column_name, idname, idname, table_name, indexed_column_name, table_name,
+			table_name, indexed_column_name, function);
+#else
+	ZBX_UNUSED(sql_alloc);
+	ZBX_UNUSED(sql_offset);
+	ZBX_UNUSED(table_name);
+	ZBX_UNUSED(original_column_name);
+	ZBX_UNUSED(indexed_column_name);
+	ZBX_UNUSED(function);
+	ZBX_UNUSED(idname);
+#endif
+	if (ZBX_DB_OK <= DBexecute("%s", sql))
+		ret = SUCCEED;
+
+	zbx_free(sql);
+
+	return ret;
+}
+
+int	zbx_dbupgrade_attach_trigger_with_function_on_update(const char *table_name,
+		const char *original_column_name, const char *indexed_column_name, const char *function,
+		const char *idname)
+{
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+#ifdef HAVE_ORACLE
+	ZBX_UNUSED(idname);
+
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create trigger %s_%s_update\n"
+			"before update on %s for each row\n"
+			"begin\n"
+				"if :new.%s<>:old.%s\n"
+				"then\n"
+					":new.%s:=%s(:new.%s);\n"
+				"end if;\n"
+			"end;",
+			table_name, indexed_column_name, table_name, original_column_name,
+			original_column_name, indexed_column_name, function, original_column_name);
+#elif HAVE_MYSQL
+	ZBX_UNUSED(idname);
+
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create trigger %s_%s_update\n"
+			"before update on %s for each row\n"
+			"begin\n"
+				"if new.%s<>old.%s\n"
+				"then\n"
+					"set new.%s=%s(new.%s);\n"
+				"end if;\n"
+			"end",
+			table_name, indexed_column_name, table_name, original_column_name,
+			original_column_name, indexed_column_name, function, original_column_name);
+#elif defined(HAVE_POSTGRESQL)
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"create or replace function %s_%s_%s()\n"
+			"returns trigger language plpgsql AS $func$\n"
+			"begin\n"
+				"update %s set %s=%s(%s)\n"
+				"where %s=new.%s;\n"
+				"return null;\n"
+			"end $func$;\n"
+
+			"create trigger %s_%s_update after update of %s on %s\n"
+				"for each row execute function %s_%s_%s();",
+			table_name, indexed_column_name, function, table_name, indexed_column_name, function,
+			original_column_name, idname, idname, table_name, indexed_column_name,
+			original_column_name, table_name, table_name, indexed_column_name, function);
+#else
+	ZBX_UNUSED(sql_alloc);
+	ZBX_UNUSED(sql_offset);
+#endif
+	if (ZBX_DB_OK <= DBexecute("%s", sql))
+		ret = SUCCEED;
+
+	zbx_free(sql);
+
+	return ret;
+}
+#endif
