@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,13 +17,13 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "zbxalgo.h"
+#include "diag.h"
+
 #include "dbcache.h"
 #include "preproc.h"
 #include "zbxdiag.h"
-#include "diag.h"
 #include "log.h"
+#include "mutexs.h"
 
 void	diag_map_free(zbx_diag_map_t *map)
 {
@@ -32,8 +32,6 @@ void	diag_map_free(zbx_diag_map_t *map)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_parse_request                                               *
  *                                                                            *
  * Purpose: parse diagnostic section request having json format               *
  *          {"stats":[<field1>,<field2>,...], "top":{<field1>:<limit1>,...}}  *
@@ -132,8 +130,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_mem_stats                                               *
  *                                                                            *
  * Purpose: add memory statistics to the json data                            *
  *                                                                            *
@@ -184,8 +180,6 @@ void	diag_add_mem_stats(struct zbx_json *json, const char *name, const zbx_mem_s
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_compare_pair_second_desc                                    *
  *                                                                            *
  * Purpose: compare uint64 pairs by second value for descending sorting       *
  *                                                                            *
@@ -203,8 +197,6 @@ static int	diag_compare_pair_second_desc(const void *d1, const void *d2)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_historycahe_add_items                                       *
  *                                                                            *
  * Purpose: add history cache items diagnostic statistics to json             *
  *                                                                            *
@@ -228,8 +220,6 @@ static void	diag_historycache_add_items(struct zbx_json *json, const char *field
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_historycache_info                                       *
  *                                                                            *
  * Purpose: add requested history cache diagnostic information to json data   *
  *                                                                            *
@@ -348,8 +338,6 @@ int	diag_add_historycache_info(const struct zbx_json_parse *jp, struct zbx_json
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_preproc_items                                           *
  *                                                                            *
  * Purpose: add item top list to output json                                  *
  *                                                                            *
@@ -379,8 +367,6 @@ static void	diag_add_preproc_items(struct zbx_json *json, const char *field, con
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_preproc_info                                            *
  *                                                                            *
  * Purpose: add requested preprocessing diagnostic information to json data   *
  *                                                                            *
@@ -413,19 +399,29 @@ int	diag_add_preproc_info(const struct zbx_json_parse *jp, struct zbx_json *json
 
 		if (0 != (fields & ZBX_DIAG_PREPROC_SIMPLE))
 		{
-			int	values_num, values_preproc_num;
+			int	total, queued, processing, done, pending;
 
 			time1 = zbx_time();
-			if (FAIL == (ret = zbx_preprocessor_get_diag_stats(&values_num, &values_preproc_num, error)))
+			if (FAIL == (ret = zbx_preprocessor_get_diag_stats(&total, &queued, &processing, &done,
+					&pending, error)))
+			{
 				goto out;
+			}
 
 			time2 = zbx_time();
 			time_total += time2 - time1;
 
 			if (0 != (fields & ZBX_DIAG_PREPROC_VALUES))
-				zbx_json_addint64(json, "values", values_num);
+			{
+				zbx_json_addint64(json, "values", total);
+				zbx_json_addint64(json, "done", done);
+			}
 			if (0 != (fields & ZBX_DIAG_PREPROC_VALUES_PREPROC))
-				zbx_json_addint64(json, "preproc.values", values_preproc_num);
+			{
+				zbx_json_addint64(json, "queued", queued);
+				zbx_json_addint64(json, "processing", processing);
+				zbx_json_addint64(json, "pending", pending);
+			}
 		}
 
 		if (0 != tops.values_num)
@@ -438,13 +434,19 @@ int	diag_add_preproc_info(const struct zbx_json_parse *jp, struct zbx_json *json
 			{
 				zbx_diag_map_t	*map = (zbx_diag_map_t *)tops.values[i];
 
-				if (0 == strcmp(map->name, "values"))
+				if (0 == strcmp(map->name, "values") || 0 == strcmp(map->name, "oldest.preproc.values"))
 				{
 					zbx_vector_ptr_t	items;
 
 					zbx_vector_ptr_create(&items);
 					time1 = zbx_time();
-					if (FAIL == (ret = zbx_preprocessor_get_top_items(map->value, &items, error)))
+					if (0 == strcmp(map->name, "values"))
+						ret = zbx_preprocessor_get_top_items(map->value, &items, error);
+					else
+						ret = zbx_preprocessor_get_top_oldest_preproc_items(map->value, &items,
+								error);
+
+					if (FAIL == ret)
 					{
 						zbx_vector_ptr_destroy(&items);
 						goto out;
@@ -486,8 +488,6 @@ static void	zbx_json_addhex(struct zbx_json *j, const char *name, zbx_uint64_t v
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_locks_info                                              *
  *                                                                            *
  * Purpose: add requested locks diagnostic information to json data           *
  *                                                                            *
@@ -500,15 +500,16 @@ void	diag_add_locks_info(struct zbx_json *json)
 #ifdef HAVE_VMINFO_T_UPDATES
 	const char	*names[ZBX_MUTEX_COUNT] = {"ZBX_MUTEX_LOG", "ZBX_MUTEX_CACHE", "ZBX_MUTEX_TRENDS",
 				"ZBX_MUTEX_CACHE_IDS", "ZBX_MUTEX_SELFMON", "ZBX_MUTEX_CPUSTATS", "ZBX_MUTEX_DISKSTATS",
-				"ZBX_MUTEX_ITSERVICES", "ZBX_MUTEX_VALUECACHE", "ZBX_MUTEX_VMWARE", "ZBX_MUTEX_SQLITE3",
-				"ZBX_MUTEX_PROCSTAT", "ZBX_MUTEX_PROXY_HISTORY", "ZBX_MUTEX_KSTAT"};
+				"ZBX_MUTEX_VALUECACHE", "ZBX_MUTEX_VMWARE", "ZBX_MUTEX_SQLITE3",
+				"ZBX_MUTEX_PROCSTAT", "ZBX_MUTEX_PROXY_HISTORY", "ZBX_MUTEX_KSTAT", "ZBX_MUTEX_MODBUS",
+				"ZBX_MUTEX_TREND_FUNC"};
 #else
 	const char	*names[ZBX_MUTEX_COUNT] = {"ZBX_MUTEX_LOG", "ZBX_MUTEX_CACHE", "ZBX_MUTEX_TRENDS",
 				"ZBX_MUTEX_CACHE_IDS", "ZBX_MUTEX_SELFMON", "ZBX_MUTEX_CPUSTATS", "ZBX_MUTEX_DISKSTATS",
-				"ZBX_MUTEX_ITSERVICES", "ZBX_MUTEX_VALUECACHE", "ZBX_MUTEX_VMWARE", "ZBX_MUTEX_SQLITE3",
-				"ZBX_MUTEX_PROCSTAT", "ZBX_MUTEX_PROXY_HISTORY"};
+				"ZBX_MUTEX_VALUECACHE", "ZBX_MUTEX_VMWARE", "ZBX_MUTEX_SQLITE3",
+				"ZBX_MUTEX_PROCSTAT", "ZBX_MUTEX_PROXY_HISTORY", "ZBX_MUTEX_MODBUS",
+				"ZBX_MUTEX_TREND_FUNC"};
 #endif
-
 	zbx_json_addarray(json, ZBX_DIAG_LOCKS);
 
 	for (i = 0; i < ZBX_MUTEX_COUNT; i++)
@@ -522,12 +523,18 @@ void	diag_add_locks_info(struct zbx_json *json)
 	zbx_json_addhex(json, "ZBX_RWLOCK_CONFIG", (zbx_uint64_t)zbx_rwlock_addr_get(ZBX_RWLOCK_CONFIG));
 	zbx_json_close(json);
 
+	zbx_json_addobject(json, NULL);
+	zbx_json_addhex(json, "ZBX_RWLOCK_CONFIG_HISTORY", (zbx_uint64_t)zbx_rwlock_addr_get(ZBX_RWLOCK_CONFIG_HISTORY));
+	zbx_json_close(json);
+
+	zbx_json_addobject(json, NULL);
+	zbx_json_addhex(json, "ZBX_RWLOCK_VALUECACHE", (zbx_uint64_t)zbx_rwlock_addr_get(ZBX_RWLOCK_VALUECACHE));
+	zbx_json_close(json);
+
 	zbx_json_close(json);
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_diag_get_info                                                *
  *                                                                            *
  * Purpose: get diagnostic information                                        *
  *                                                                            *
@@ -571,8 +578,6 @@ out:
 #define ZBX_DIAG_DEFAULT_TOP_LIMIT	25
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_add_section_request                                         *
  *                                                                            *
  * Purpose: add default diagnostic section request                            *
  *                                                                            *
@@ -603,8 +608,6 @@ static void	diag_add_section_request(struct zbx_json *j, const char *section, ..
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_prepare_default_request                                     *
  *                                                                            *
  * Purpose: prepare default diagnostic request for all sections               *
  *                                                                            *
@@ -618,7 +621,7 @@ static void	diag_prepare_default_request(struct zbx_json *j, unsigned int flags)
 		diag_add_section_request(j, ZBX_DIAG_VALUECACHE, "values", "request.values", NULL);
 
 	if (0 != (flags & (1 << ZBX_DIAGINFO_PREPROCESSING)))
-		diag_add_section_request(j, ZBX_DIAG_PREPROCESSING, "values", NULL);
+		diag_add_section_request(j, ZBX_DIAG_PREPROCESSING, "values", "oldest.preproc.values", NULL);
 
 	if (0 != (flags & (1 << ZBX_DIAGINFO_LLD)))
 		diag_add_section_request(j, ZBX_DIAG_LLD, "values", NULL);
@@ -631,8 +634,6 @@ static void	diag_prepare_default_request(struct zbx_json *j, unsigned int flags)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_get_simple_values                                           *
  *                                                                            *
  * Purpose: extract simple values in format <key1>:<value1> <key2>:<value2>...*
  *          from the specified json location                                  *
@@ -653,7 +654,8 @@ static void	diag_get_simple_values(const struct zbx_json_parse *jp, char **msg)
 	{
 		if (FAIL == zbx_json_brackets_open(pnext, &jp_value))
 		{
-			zbx_json_decodevalue_dyn(pnext, &value, &value_alloc, &type);
+			if (NULL == zbx_json_decodevalue_dyn(pnext, &value, &value_alloc, &type))
+				type = ZBX_JSON_TYPE_NULL;
 
 			if (0 != msg_offset)
 				zbx_chrcpy_alloc(msg, &msg_alloc, &msg_offset, ' ');
@@ -667,8 +669,6 @@ static void	diag_get_simple_values(const struct zbx_json_parse *jp, char **msg)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_memory_info                                             *
  *                                                                            *
  * Purpose: log shared memory information                                     *
  *                                                                            *
@@ -677,7 +677,8 @@ static void	diag_get_simple_values(const struct zbx_json_parse *jp, char **msg)
  *             path  - [OUT] the json path to the memory data                 *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_memory_info(struct zbx_json_parse *jp, const char *field, const char *path)
+static void	diag_log_memory_info(struct zbx_json_parse *jp, const char *field, const char *path, char **out,
+		size_t *out_alloc, size_t *out_offset)
 {
 	struct zbx_json_parse	jp_memory, jp_size, jp_chunks;
 	char			*msg = NULL;
@@ -685,11 +686,11 @@ static void	diag_log_memory_info(struct zbx_json_parse *jp, const char *field, c
 	if (FAIL == zbx_json_open_path(jp, path, &jp_memory))
 		return;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s:", field);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s:", field);
 	if (SUCCEED == zbx_json_brackets_by_name(&jp_memory, "size", &jp_size))
 	{
 		diag_get_simple_values(&jp_size, &msg);
-		zabbix_log(LOG_LEVEL_INFORMATION, "  size: %s", msg);
+		zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "  size: %s", msg);
 		zbx_free(msg);
 	}
 
@@ -698,21 +699,22 @@ static void	diag_log_memory_info(struct zbx_json_parse *jp, const char *field, c
 		struct zbx_json_parse	jp_buckets, jp_bucket;
 
 		diag_get_simple_values(&jp_chunks, &msg);
-		zabbix_log(LOG_LEVEL_INFORMATION, "  chunks: %s", msg);
+		zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "  chunks: %s", msg);
 		zbx_free(msg);
 
 		if (SUCCEED == zbx_json_brackets_by_name(&jp_chunks, "buckets", &jp_buckets))
 		{
 			const char	*pnext;
 
-			zabbix_log(LOG_LEVEL_INFORMATION, "    buckets:");
+			zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "    buckets:");
 
 			for (pnext = NULL; NULL != (pnext = zbx_json_next(&jp_buckets, pnext));)
 			{
 				if (SUCCEED == zbx_json_brackets_open(pnext, &jp_bucket))
 				{
 					diag_get_simple_values(&jp_bucket, &msg);
-					zabbix_log(LOG_LEVEL_INFORMATION, "      %s", msg);
+					zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "      %s",
+							msg);
 					zbx_free(msg);
 				}
 			}
@@ -721,17 +723,19 @@ static void	diag_log_memory_info(struct zbx_json_parse *jp, const char *field, c
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_top_view                                                *
  *                                                                            *
  * Purpose: log top view                                                      *
  *                                                                            *
- * Parameters: jp    - [IN] the section json                                  *
- *             field - [OUT] the top field name                               *
- *             path  - [OUT] the json path to the top view                    *
+ * Parameters: jp         - [IN] the section json                             *
+ *             field      - [OUT] the top field name                          *
+ *             path       - [OUT] the json path to the top view               *
+ *             out        - [OUT] the output buffer (optional)                *
+ *             out_alloc  - [OUT] the output buffer size                      *
+ *             out_offset - [OUT] the output buffer offset                    *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_top_view(struct zbx_json_parse *jp, const char *field, const char *path)
+static void	diag_log_top_view(struct zbx_json_parse *jp, const char *field, const char *path,
+		char **out, size_t *out_alloc, size_t *out_offset)
 {
 	struct zbx_json_parse	jp_top, jp_row;
 	const char		*pnext;
@@ -744,155 +748,146 @@ static void	diag_log_top_view(struct zbx_json_parse *jp, const char *field, cons
 	else if (FAIL == zbx_json_open_path(jp, path, &jp_top))
 		return;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s:", field);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s:", field);
 
 	for (pnext = NULL; NULL != (pnext = zbx_json_next(&jp_top, pnext));)
 	{
 		if (SUCCEED == zbx_json_brackets_open(pnext, &jp_row))
 		{
 			diag_get_simple_values(&jp_row, &msg);
-			zabbix_log(LOG_LEVEL_INFORMATION, "  %s", msg);
+			zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "  %s", msg);
 			zbx_free(msg);
 		}
 	}
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_history_cache                                           *
  *                                                                            *
  * Purpose: log history cache diagnostic information                          *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_history_cache(struct zbx_json_parse *jp)
+static void	diag_log_history_cache(struct zbx_json_parse *jp, char **out, size_t *out_alloc, size_t *out_offset)
 {
 	char	*msg = NULL;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "== history cache diagnostic information ==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "== history cache diagnostic information ==");
 
 	diag_get_simple_values(jp, &msg);
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s", msg);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s", msg);
 	zbx_free(msg);
 
-	diag_log_memory_info(jp, "memory.data", "$.memory.data");
-	diag_log_memory_info(jp, "memory.index", "$.memory.index");
+	diag_log_memory_info(jp, "memory.data", "$.memory.data", out, out_alloc, out_offset);
+	diag_log_memory_info(jp, "memory.index", "$.memory.index", out, out_alloc, out_offset);
 
-	diag_log_top_view(jp, "top.values", "$.top.values");
+	diag_log_top_view(jp, "top.values", "$.top.values", out, out_alloc, out_offset);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "==");
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_value_cache                                             *
  *                                                                            *
  * Purpose: log value cache diagnostic information                            *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_value_cache(struct zbx_json_parse *jp)
+static void	diag_log_value_cache(struct zbx_json_parse *jp, char **out, size_t *out_alloc, size_t *out_offset)
 {
 	char	*msg = NULL;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "== value cache diagnostic information ==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "== value cache diagnostic information ==");
 
 	diag_get_simple_values(jp, &msg);
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s", msg);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s", msg);
 	zbx_free(msg);
 
-	diag_log_memory_info(jp, "memory", "$.memory");
+	diag_log_memory_info(jp, "memory", "$.memory", out, out_alloc, out_offset);
 
-	diag_log_top_view(jp, "top.values", "$.top.values");
-	diag_log_top_view(jp, "top.request.values", "$.top['request.values']");
+	diag_log_top_view(jp, "top.values", "$.top.values", out, out_alloc, out_offset);
+	diag_log_top_view(jp, "top.request.values", "$.top['request.values']", out, out_alloc, out_offset);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "==");
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_preprocessing                                           *
  *                                                                            *
  * Purpose: log preprocessing diagnostic information                          *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_preprocessing(struct zbx_json_parse *jp)
+static void	diag_log_preprocessing(struct zbx_json_parse *jp, char **out, size_t *out_alloc, size_t *out_offset)
 {
 	char	*msg = NULL;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "== preprocessing diagnostic information ==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "== preprocessing diagnostic information ==");
 
 	diag_get_simple_values(jp, &msg);
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s", msg);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s", msg);
 	zbx_free(msg);
 
-	diag_log_top_view(jp, "top.values", "$.top.values");
+	diag_log_top_view(jp, "top.values", "$.top.values", out, out_alloc, out_offset);
+	diag_log_top_view(jp, "top.oldest.preproc.values", "$.top['oldest.preproc.values']", out, out_alloc, out_offset);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "==");
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_lld                                                     *
  *                                                                            *
  * Purpose: log LLD diagnostic information                                    *
  *                                                                            *
  ******************************************************************************/
 
-static void	diag_log_lld(struct zbx_json_parse *jp)
+static void	diag_log_lld(struct zbx_json_parse *jp, char **out, size_t *out_alloc, size_t *out_offset)
 {
 	char	*msg = NULL;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "== LLD diagnostic information ==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "== LLD diagnostic information ==");
 
 	diag_get_simple_values(jp, &msg);
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s", msg);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s", msg);
 	zbx_free(msg);
 
-	diag_log_top_view(jp, "top.values", "$.top.values");
+	diag_log_top_view(jp, "top.values", "$.top.values", out, out_alloc, out_offset);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "==");
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: diag_log_alerting                                                *
  *                                                                            *
  * Purpose: log alerting diagnostic information                               *
  *                                                                            *
  ******************************************************************************/
-static void	diag_log_alerting(struct zbx_json_parse *jp)
+static void	diag_log_alerting(struct zbx_json_parse *jp, char **out, size_t *out_alloc, size_t *out_offset)
 {
 	char	*msg = NULL;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "== alerting diagnostic information ==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "== alerting diagnostic information ==");
 
 	diag_get_simple_values(jp, &msg);
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s", msg);
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "%s", msg);
 	zbx_free(msg);
 
-	diag_log_top_view(jp, "media.alerts", "$.top['media.alerts']");
-	diag_log_top_view(jp, "source.alerts", "$.top['source.alerts']");
+	diag_log_top_view(jp, "media.alerts", "$.top['media.alerts']", out, out_alloc, out_offset);
+	diag_log_top_view(jp, "source.alerts", "$.top['source.alerts']", out, out_alloc, out_offset);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "==");
+	zbx_strlog_alloc(LOG_LEVEL_INFORMATION, out, out_alloc, out_offset, "==");
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_diag_log_info                                                *
  *                                                                            *
  * Purpose: log diagnostic information                                        *
  *                                                                            *
  * Parameters: flags - [IN] flags describing section to log                   *
  *                                                                            *
  ******************************************************************************/
-void	zbx_diag_log_info(unsigned int flags)
+void	zbx_diag_log_info(unsigned int flags, char **result)
 {
 	struct zbx_json		j;
 	struct zbx_json_parse	jp;
 	char			*info = NULL;
+	size_t			result_alloc = 0, result_offset = 0;
 
 	zbx_json_init(&j, 1024);
 
 	diag_prepare_default_request(&j, flags);
+
 	if (FAIL == zbx_json_open(j.buffer, &jp))
 	{
 		THIS_SHOULD_NEVER_HAPPEN;
@@ -920,27 +915,31 @@ void	zbx_diag_log_info(unsigned int flags)
 			}
 
 			if (0 == strcmp(section, ZBX_DIAG_HISTORYCACHE))
-				diag_log_history_cache(&jp_section);
+				diag_log_history_cache(&jp_section, result, &result_alloc, &result_offset);
 			else if (0 == strcmp(section, ZBX_DIAG_VALUECACHE))
-				diag_log_value_cache(&jp_section);
+				diag_log_value_cache(&jp_section, result, &result_alloc, &result_offset);
 			else if (0 == strcmp(section, ZBX_DIAG_PREPROCESSING))
-				diag_log_preprocessing(&jp_section);
+				diag_log_preprocessing(&jp_section, result, &result_alloc, &result_offset);
 			else if (0 == strcmp(section, ZBX_DIAG_LLD))
-				diag_log_lld(&jp_section);
+				diag_log_lld(&jp_section, result, &result_alloc, &result_offset);
 			else if (0 == strcmp(section, ZBX_DIAG_ALERTING))
-				diag_log_alerting(&jp_section);
+				diag_log_alerting(&jp_section, result, &result_alloc, &result_offset);
 			else if (0 == strcmp(section, ZBX_DIAG_LOCKS))
 			{
-				zabbix_log(LOG_LEVEL_INFORMATION, "== locks diagnostic information ==");
-				diag_log_top_view(&jp_section, ZBX_DIAG_LOCKS, NULL);
-				zabbix_log(LOG_LEVEL_INFORMATION, "==");
+				zbx_strlog_alloc(LOG_LEVEL_INFORMATION, result, &result_alloc, &result_offset,
+						"== locks diagnostic information ==");
+				diag_log_top_view(&jp_section, ZBX_DIAG_LOCKS, NULL, result, &result_alloc,
+						&result_offset);
+				zbx_strlog_alloc(LOG_LEVEL_INFORMATION, result, &result_alloc, &result_offset, "==");
 			}
 		}
 	}
 	else
-		zabbix_log(LOG_LEVEL_INFORMATION, "cannot obtain diagnostic information: %s", info);
+	{
+		zbx_strlog_alloc(LOG_LEVEL_INFORMATION, result, &result_alloc, &result_offset,
+				"cannot obtain diagnostic information: %s", info);
+	}
 out:
 	zbx_free(info);
 	zbx_json_free(&j);
 }
-
