@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,22 +17,18 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "mutexs.h"
-#include "ipc.h"
-#include "log.h"
-
 #include "memalloc.h"
 
+#include "common.h"
+#include "log.h"
+
 /******************************************************************************
  *                                                                            *
  *                     Some information on memory layout                      *
  *                  ---------------------------------------                   *
  *                                                                            *
- *                                                                            *
  * (*) chunk: a contiguous piece of memory that is either free or used        *
  *                                                                            *
- *                                                                            *
  *                    +-------- size of + --------------+                     *
  *                    |       (8 bytes) |               |                     *
  *                    |                 v               |                     *
@@ -48,7 +44,6 @@
  *                                                                            *
  *                     8-aligned               8-aligned                      *
  *                                                                            *
- *                                                                            *
  *     when a chunk is used, `size' fields have MEM_FLG_USED bit set          *
  *                                                                            *
  *     when a chunk is free, the first 2 * ZBX_PTR_SIZE bytes of allocatable  *
@@ -62,12 +57,10 @@
  *           (when freeing a chunk, we can quickly see if the previous        *
  *           and next chunks are free, those will not have MEM_FLG_USED)      *
  *                                                                            *
- *                                                                            *
  * (*) free chunks are stored in doubly-linked lists according to their sizes *
  *                                                                            *
  *     a typical situation is thus as follows (1 used chunk, 2 free chunks)   *
  *                                                                            *
- *                                                                            *
  *  +--------------------------- shared memory ----------------------------+  *
  *  |                         (can be misaligned)                          |  *
  *  |                                                                      |  *
@@ -91,9 +84,6 @@
  *                                                                            *
  ******************************************************************************/
 
-#define LOCK_INFO	if (info->use_lock) zbx_mutex_lock(&info->mem_lock)
-#define UNLOCK_INFO	if (info->use_lock) zbx_mutex_unlock(&info->mem_lock)
-
 static void	*ALIGN4(void *ptr);
 static void	*ALIGN8(void *ptr);
 static void	*ALIGNPTR(void *ptr);
@@ -128,12 +118,6 @@ static void	__mem_free(zbx_mem_info_t *info, void *ptr);
 #define MEM_MIN_SIZE		__UINT64_C(128)
 #define MEM_MAX_SIZE		__UINT64_C(0x1000000000)	/* 64 GB */
 
-#define MEM_MIN_ALLOC	24	/* should be a multiple of 8 and at least (2 * ZBX_PTR_SIZE) */
-
-#define MEM_MIN_BUCKET_SIZE	MEM_MIN_ALLOC
-#define MEM_MAX_BUCKET_SIZE	256 /* starting from this size all free chunks are put into the same bucket */
-#define MEM_BUCKET_COUNT	((MEM_MAX_BUCKET_SIZE - MEM_MIN_BUCKET_SIZE) / 8 + 1)
-
 /* helper functions */
 
 static void	*ALIGN4(void *ptr)
@@ -287,13 +271,15 @@ static void	*__mem_malloc(zbx_mem_info_t *info, zbx_uint64_t size)
 		{
 			if (NULL == chunk)
 			{
-				zabbix_log(LOG_LEVEL_CRIT, "__mem_malloc: skipped %d asked %u skip_min %u skip_max %u",
+				zabbix_log(LOG_LEVEL_CRIT, "__mem_malloc: skipped %d asked " ZBX_FS_UI64 " skip_min "
+						ZBX_FS_UI64 " skip_max " ZBX_FS_UI64,
 						counter, size, skip_min, skip_max);
 			}
 			else if (counter >= 100)
 			{
-				zabbix_log(LOG_LEVEL_DEBUG, "__mem_malloc: skipped %d asked %u skip_min %u skip_max %u"
-						" size %u", counter, size, skip_min, skip_max, CHUNK_SIZE(chunk));
+				zabbix_log(LOG_LEVEL_DEBUG, "__mem_malloc: skipped %d asked " ZBX_FS_UI64 " skip_min "
+						ZBX_FS_UI64 " skip_max " ZBX_FS_UI64 " size " ZBX_FS_UI64, counter,
+						size, skip_min, skip_max, CHUNK_SIZE(chunk));
 			}
 		}
 	}
@@ -458,7 +444,7 @@ static void	*__mem_realloc(zbx_mem_info_t *info, void *old, zbx_uint64_t size)
 		if (NULL == (new_chunk = __mem_malloc(info, size)))
 		{
 			THIS_SHOULD_NEVER_HAPPEN;
-			exit(FAIL);
+			exit(EXIT_FAILURE);
 		}
 
 		memcpy((char *)new_chunk + MEM_SIZE_FIELD, tmp, chunk_size);
@@ -540,89 +526,78 @@ static void	__mem_free(zbx_mem_info_t *info, void *ptr)
 
 /* public memory interface */
 
-void	zbx_mem_create(zbx_mem_info_t **info, key_t shm_key, int lock_name, zbx_uint64_t size,
-		const char *descr, const char *param, int allow_oom)
+int	zbx_mem_create(zbx_mem_info_t **info, zbx_uint64_t size, const char *descr, const char *param, int allow_oom,
+		char **error)
 {
-	const char	*__function_name = "zbx_mem_create";
-
-	int		shm_id, index;
-	void		*base;
+	int	shm_id, index, ret = FAIL;
+	void	*base;
 
 	descr = ZBX_NULL2STR(descr);
 	param = ZBX_NULL2STR(param);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() descr:'%s' param:'%s' size:" ZBX_FS_SIZE_T,
-			__function_name, descr, param, (zbx_fs_size_t)size);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() param:'%s' size:" ZBX_FS_SIZE_T, __func__, param, (zbx_fs_size_t)size);
 
 	/* allocate shared memory */
 
 	if (4 != ZBX_PTR_SIZE && 8 != ZBX_PTR_SIZE)
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "failed assumption about pointer size (" ZBX_FS_SIZE_T " not in {4, 8})",
+		*error = zbx_dsprintf(*error, "failed assumption about pointer size (" ZBX_FS_SIZE_T " not in {4, 8})",
 				(zbx_fs_size_t)ZBX_PTR_SIZE);
-		exit(FAIL);
+		goto out;
 	}
 
 	if (!(MEM_MIN_SIZE <= size && size <= MEM_MAX_SIZE))
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "requested size " ZBX_FS_SIZE_T " not within bounds [" ZBX_FS_UI64
-				" <= size <= " ZBX_FS_UI64 "]", (zbx_fs_size_t)size, MEM_MIN_SIZE, MEM_MAX_SIZE);
-		exit(FAIL);
+		*error = zbx_dsprintf(*error, "requested size " ZBX_FS_UI64 " not within bounds [" ZBX_FS_UI64
+				" <= size <= " ZBX_FS_UI64 "]", size, MEM_MIN_SIZE, MEM_MAX_SIZE);
+		goto out;
 	}
 
-	if (-1 == (shm_id = zbx_shmget(shm_key, size)))
+	if (-1 == (shm_id = shmget(IPC_PRIVATE, size, 0600)))
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "cannot allocate shared memory for %s", descr);
-		exit(FAIL);
+		*error = zbx_dsprintf(*error, "cannot get private shared memory of size " ZBX_FS_SIZE_T " for %s: %s",
+				(zbx_fs_size_t)size, descr, zbx_strerror(errno));
+		goto out;
 	}
 
 	if ((void *)(-1) == (base = shmat(shm_id, NULL, 0)))
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "cannot attach shared memory for %s: %s", descr, zbx_strerror(errno));
-		exit(FAIL);
+		*error = zbx_dsprintf(*error, "cannot attach shared memory for %s: %s", descr, zbx_strerror(errno));
+		goto out;
 	}
 
+	if (-1 == shmctl(shm_id, IPC_RMID, NULL))
+		zbx_error("cannot mark shared memory %d for destruction: %s", shm_id, zbx_strerror(errno));
+
+	ret = SUCCEED;
+
 	/* allocate zbx_mem_info_t structure, its buckets, and description inside shared memory */
 
-	*info = ALIGN8(base);
+	*info = (zbx_mem_info_t *)ALIGN8(base);
+	(*info)->base = base;
 	(*info)->shm_id = shm_id;
 	(*info)->orig_size = size;
 	size -= (char *)(*info + 1) - (char *)base;
 
 	base = (void *)(*info + 1);
 
-	(*info)->buckets = ALIGNPTR(base);
+	(*info)->buckets = (void **)ALIGNPTR(base);
 	memset((*info)->buckets, 0, MEM_BUCKET_COUNT * ZBX_PTR_SIZE);
 	size -= (char *)((*info)->buckets + MEM_BUCKET_COUNT) - (char *)base;
 	base = (void *)((*info)->buckets + MEM_BUCKET_COUNT);
 
-	zbx_strlcpy(base, descr, size);
-	(*info)->mem_descr = base;
+	zbx_strlcpy((char *)base, descr, size);
+	(*info)->mem_descr = (char *)base;
 	size -= strlen(descr) + 1;
 	base = (void *)((char *)base + strlen(descr) + 1);
 
-	zbx_strlcpy(base, param, size);
-	(*info)->mem_param = base;
+	zbx_strlcpy((char *)base, param, size);
+	(*info)->mem_param = (char *)base;
 	size -= strlen(param) + 1;
 	base = (void *)((char *)base + strlen(param) + 1);
 
 	(*info)->allow_oom = allow_oom;
 
-	/* allocate mutex */
-
-	if (ZBX_NO_MUTEX != lock_name)
-	{
-		(*info)->use_lock = 1;
-
-		if (ZBX_MUTEX_ERROR == zbx_mutex_create_force(&((*info)->mem_lock), lock_name))
-		{
-			zabbix_log(LOG_LEVEL_CRIT, "cannot create mutex for %s", descr);
-			exit(FAIL);
-		}
-	}
-	else
-		(*info)->use_lock = 0;
-
 	/* prepare shared memory for further allocation by creating one big chunk */
 	(*info)->lo_bound = ALIGN8(base);
 	(*info)->hi_bound = ALIGN8((char *)base + size - 8);
@@ -640,67 +615,54 @@ void	zbx_mem_create(zbx_mem_info_t **info, key_t shm_key, int lock_name, zbx_uin
 	(*info)->free_size = (*info)->total_size;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "valid user addresses: [%p, %p] total size: " ZBX_FS_SIZE_T,
-			(char *)(*info)->lo_bound + MEM_SIZE_FIELD,
-			(char *)(*info)->hi_bound - MEM_SIZE_FIELD,
+			(void *)((char *)(*info)->lo_bound + MEM_SIZE_FIELD),
+			(void *)((char *)(*info)->hi_bound - MEM_SIZE_FIELD),
 			(zbx_fs_size_t)(*info)->total_size);
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	return ret;
 }
 
 void	zbx_mem_destroy(zbx_mem_info_t *info)
 {
-	const char	*__function_name = "zbx_mem_destroy";
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() descr:'%s'", __function_name, info->mem_descr);
-
-	if (info->use_lock)
-		zbx_mutex_destroy(&info->mem_lock);
-
-	if (-1 == shmctl(info->shm_id, IPC_RMID, 0))
-	{
-		zabbix_log(LOG_LEVEL_WARNING, "cannot remove shared memory for %s: %s",
-				info->mem_descr, zbx_strerror(errno));
-	}
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	(void)shmdt(info->base);
 }
 
 void	*__zbx_mem_malloc(const char *file, int line, zbx_mem_info_t *info, const void *old, size_t size)
 {
-	const char	*__function_name = "zbx_mem_malloc";
-
-	void		*chunk;
+	void	*chunk;
 
 	if (NULL != old)
 	{
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): allocating already allocated memory",
-				file, line, __function_name);
-		exit(FAIL);
+				file, line, __func__);
+		exit(EXIT_FAILURE);
 	}
 
 	if (0 == size || size > MEM_MAX_SIZE)
 	{
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): asking for a bad number of bytes (" ZBX_FS_SIZE_T
-				")", file, line, __function_name, (zbx_fs_size_t)size);
-		exit(FAIL);
+				")", file, line, __func__, (zbx_fs_size_t)size);
+		exit(EXIT_FAILURE);
 	}
 
-	LOCK_INFO;
-
 	chunk = __mem_malloc(info, size);
 
-	UNLOCK_INFO;
-
 	if (NULL == chunk)
 	{
 		if (1 == info->allow_oom)
 			return NULL;
 
+		zbx_mem_dump_stats(LOG_LEVEL_CRIT, info);
+		zbx_backtrace();
+
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): out of memory (requested " ZBX_FS_SIZE_T " bytes)",
-				file, line, __function_name, (zbx_fs_size_t)size);
+				file, line, __func__, (zbx_fs_size_t)size);
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): please increase %s configuration parameter",
-				file, line, __function_name, info->mem_param);
-		exit(FAIL);
+				file, line, __func__, info->mem_param);
+
+		exit(EXIT_FAILURE);
 	}
 
 	return (void *)((char *)chunk + MEM_SIZE_FIELD);
@@ -708,36 +670,34 @@ void	*__zbx_mem_malloc(const char *file, int line, zbx_mem_info_t *info, const v
 
 void	*__zbx_mem_realloc(const char *file, int line, zbx_mem_info_t *info, void *old, size_t size)
 {
-	const char	*__function_name = "zbx_mem_realloc";
-
-	void		*chunk;
+	void	*chunk;
 
 	if (0 == size || size > MEM_MAX_SIZE)
 	{
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): asking for a bad number of bytes (" ZBX_FS_SIZE_T
-				")", file, line, __function_name, (zbx_fs_size_t)size);
-		exit(FAIL);
+				")", file, line, __func__, (zbx_fs_size_t)size);
+		exit(EXIT_FAILURE);
 	}
 
-	LOCK_INFO;
-
 	if (NULL == old)
 		chunk = __mem_malloc(info, size);
 	else
 		chunk = __mem_realloc(info, old, size);
 
-	UNLOCK_INFO;
-
 	if (NULL == chunk)
 	{
 		if (1 == info->allow_oom)
 			return NULL;
 
+		zbx_mem_dump_stats(LOG_LEVEL_CRIT, info);
+		zbx_backtrace();
+
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): out of memory (requested " ZBX_FS_SIZE_T " bytes)",
-				file, line, __function_name, (zbx_fs_size_t)size);
+				file, line, __func__, (zbx_fs_size_t)size);
 		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): please increase %s configuration parameter",
-				file, line, __function_name, info->mem_param);
-		exit(FAIL);
+				file, line, __func__, info->mem_param);
+
+		exit(EXIT_FAILURE);
 	}
 
 	return (void *)((char *)chunk + MEM_SIZE_FIELD);
@@ -745,31 +705,20 @@ void	*__zbx_mem_realloc(const char *file, int line, zbx_mem_info_t *info, void *
 
 void	__zbx_mem_free(const char *file, int line, zbx_mem_info_t *info, void *ptr)
 {
-	const char	*__function_name = "zbx_mem_free";
-
 	if (NULL == ptr)
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): freeing a NULL pointer",
-				file, line, __function_name);
-		exit(FAIL);
+		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] %s(): freeing a NULL pointer", file, line, __func__);
+		exit(EXIT_FAILURE);
 	}
 
-	LOCK_INFO;
-
 	__mem_free(info, ptr);
-
-	UNLOCK_INFO;
 }
 
 void	zbx_mem_clear(zbx_mem_info_t *info)
 {
-	const char	*__function_name = "zbx_mem_clear";
+	int	index;
 
-	int		index;
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
-
-	LOCK_INFO;
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	memset(info->buckets, 0, MEM_BUCKET_COUNT * ZBX_PTR_SIZE);
 	index = mem_bucket_by_size(info->total_size);
@@ -780,65 +729,82 @@ void	zbx_mem_clear(zbx_mem_info_t *info)
 	info->used_size = 0;
 	info->free_size = info->total_size;
 
-	UNLOCK_INFO;
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
-void	zbx_mem_dump_stats(zbx_mem_info_t *info)
+void	zbx_mem_get_stats(const zbx_mem_info_t *info, zbx_mem_stats_t *stats)
 {
 	void		*chunk;
-	int		index;
-	zbx_uint64_t	counter, total, total_free = 0;
-	zbx_uint64_t	min_size = __UINT64_C(0xffffffffffffffff), max_size = __UINT64_C(0);
+	int		i;
+	zbx_uint64_t	counter;
 
-	LOCK_INFO;
+	stats->free_chunks = 0;
+	stats->max_chunk_size = __UINT64_C(0);
+	stats->min_chunk_size = __UINT64_C(0xffffffffffffffff);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "=== memory statistics for %s ===", info->mem_descr);
-
-	for (index = 0; index < MEM_BUCKET_COUNT; index++)
+	for (i = 0; i < MEM_BUCKET_COUNT; i++)
 	{
 		counter = 0;
-		chunk = info->buckets[index];
+		chunk = info->buckets[i];
 
 		while (NULL != chunk)
 		{
 			counter++;
-			min_size = MIN(min_size, CHUNK_SIZE(chunk));
-			max_size = MAX(max_size, CHUNK_SIZE(chunk));
+			stats->min_chunk_size = MIN(stats->min_chunk_size, CHUNK_SIZE(chunk));
+			stats->max_chunk_size = MAX(stats->max_chunk_size, CHUNK_SIZE(chunk));
 			chunk = mem_get_next_chunk(chunk);
 		}
 
-		if (counter > 0)
-		{
-			total_free += counter;
-			zabbix_log(LOG_LEVEL_DEBUG, "free chunks of size %2s %3d bytes: %8d",
-					index == MEM_BUCKET_COUNT - 1 ? ">=" : "",
-					MEM_MIN_BUCKET_SIZE + 8 * index, counter);
-		}
+		stats->free_chunks += counter;
+		stats->chunks_num[i] = counter;
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "min chunk size: %10u bytes", min_size);
-	zabbix_log(LOG_LEVEL_DEBUG, "max chunk size: %10u bytes", max_size);
+	stats->overhead = info->total_size - info->used_size - info->free_size;
+	stats->used_chunks = stats->overhead / (2 * MEM_SIZE_FIELD) + 1 - stats->free_chunks;
+	stats->free_size = info->free_size;
+	stats->used_size = info->used_size;
+}
 
-	total = (info->total_size - info->used_size - info->free_size) / (2 * MEM_SIZE_FIELD) + 1;
-	zabbix_log(LOG_LEVEL_DEBUG, "memory of total size %u bytes fragmented into %d chunks", info->total_size, total);
-	zabbix_log(LOG_LEVEL_DEBUG, "of those, %10u bytes are in %8d free chunks", info->free_size, total_free);
-	zabbix_log(LOG_LEVEL_DEBUG, "of those, %10u bytes are in %8d used chunks", info->used_size, total - total_free);
+void	zbx_mem_dump_stats(int level, zbx_mem_info_t *info)
+{
+	zbx_mem_stats_t	stats;
+	int		i;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "================================");
+	zbx_mem_get_stats(info, &stats);
 
-	UNLOCK_INFO;
+	zabbix_log(level, "=== memory statistics for %s ===", info->mem_descr);
+
+	for (i = 0; i < MEM_BUCKET_COUNT; i++)
+	{
+		if (0 == stats.chunks_num[i])
+			continue;
+
+		zabbix_log(level, "free chunks of size %2s %3d bytes: %8u", i == MEM_BUCKET_COUNT - 1 ? ">=" : "",
+			MEM_MIN_BUCKET_SIZE + 8 * i, stats.chunks_num[i]);
+	}
+
+	zabbix_log(level, "min chunk size: %10llu bytes", (unsigned long long)stats.min_chunk_size);
+	zabbix_log(level, "max chunk size: %10llu bytes", (unsigned long long)stats.max_chunk_size);
+
+	zabbix_log(level, "memory of total size %llu bytes fragmented into %llu chunks",
+			(unsigned long long)stats.free_size + stats.used_size,
+			(unsigned long long)stats.free_chunks + stats.used_chunks);
+	zabbix_log(level, "of those, %10llu bytes are in %8llu free chunks",
+			(unsigned long long)stats.free_size, (unsigned long long)stats.free_chunks);
+	zabbix_log(level, "of those, %10llu bytes are in %8llu used chunks",
+			(unsigned long long)stats.used_size, (unsigned long long)stats.used_chunks);
+	zabbix_log(level, "of those, %10llu bytes are used by allocation overhead",
+			(unsigned long long)stats.overhead);
+
+	zabbix_log(level, "================================");
 }
 
 size_t	zbx_mem_required_size(int chunks_num, const char *descr, const char *param)
 {
-	const char	*__function_name = "zbx_mem_required_size";
-
-	size_t		size = 0;
+	size_t	size = 0;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() size:" ZBX_FS_SIZE_T " chunks_num:%d descr:'%s' param:'%s'",
-			__function_name, (zbx_fs_size_t)size, chunks_num, descr, param);
+			__func__, (zbx_fs_size_t)size, chunks_num, descr, param);
 
 	/* shared memory of what size should we allocate so that there is a guarantee */
 	/* that we will be able to get ourselves 'chunks_num' pieces of memory with a */
@@ -856,7 +822,15 @@ size_t	zbx_mem_required_size(int chunks_num, const char *descr, const char *para
 	size += (chunks_num - 1) * MEM_SIZE_FIELD * 2;	/* each additional chunk requires 16 bytes of overhead */
 	size += chunks_num * (MEM_MIN_ALLOC - 1);	/* each chunk has size of at least MEM_MIN_ALLOC bytes */
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() size:" ZBX_FS_SIZE_T, __function_name, (zbx_fs_size_t)size);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() size:" ZBX_FS_SIZE_T, __func__, (zbx_fs_size_t)size);
 
 	return size;
 }
+
+zbx_uint64_t	zbx_mem_required_chunk_size(zbx_uint64_t size)
+{
+	if (0 == size)
+		return 0;
+
+	return mem_proper_alloc_size(size) + MEM_SIZE_FIELD * 2;
+}
