@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #include "common.h"
 #include "sysinfo.h"
 
-static int	read_uint64_from_procfs(const char *path, zbx_uint64_t *value)
+static int	read_uint64_from_procfs(const char *path, int first_num, zbx_uint64_t *value)
 {
 	int	ret = SYSINFO_RET_FAIL;
 	char	line[MAX_STRING_LEN];
@@ -30,8 +30,16 @@ static int	read_uint64_from_procfs(const char *path, zbx_uint64_t *value)
 	{
 		if (NULL != fgets(line, sizeof(line), f))
 		{
-			if (1 == sscanf(line, ZBX_FS_UI64 "\n", value))
-				ret = SYSINFO_RET_OK;
+			if (1 == first_num)
+			{
+				if (1 == sscanf(line, ZBX_FS_UI64 "\t", value))
+					ret = SYSINFO_RET_OK;
+			}
+			else
+			{
+				if (1 == sscanf(line, ZBX_FS_UI64 "\n", value))
+					ret = SYSINFO_RET_OK;
+			}
 		}
 		zbx_fclose(f);
 	}
@@ -45,7 +53,7 @@ int	KERNEL_MAXFILES(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	ZBX_UNUSED(request);
 
-	if (SYSINFO_RET_FAIL == read_uint64_from_procfs("/proc/sys/fs/file-max", &value))
+	if (SYSINFO_RET_FAIL == read_uint64_from_procfs("/proc/sys/fs/file-max", 0, &value))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain data from /proc/sys/fs/file-max."));
 		return SYSINFO_RET_FAIL;
@@ -61,7 +69,7 @@ int	KERNEL_MAXPROC(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	ZBX_UNUSED(request);
 
-	if (SYSINFO_RET_FAIL == read_uint64_from_procfs("/proc/sys/kernel/pid_max", &value))
+	if (SYSINFO_RET_FAIL == read_uint64_from_procfs("/proc/sys/kernel/pid_max", 0, &value))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain data from /proc/sys/kernel/pid_max."));
 		return SYSINFO_RET_FAIL;
@@ -70,3 +78,19 @@ int	KERNEL_MAXPROC(AGENT_REQUEST *request, AGENT_RESULT *result)
 	SET_UI64_RESULT(result, value);
 	return SYSINFO_RET_OK;
 }
+
+int	KERNEL_OPENFILES(AGENT_REQUEST *request, AGENT_RESULT *result)
+{
+	zbx_uint64_t	value;
+
+	ZBX_UNUSED(request);
+
+	if (SYSINFO_RET_FAIL == read_uint64_from_procfs("/proc/sys/fs/file-nr", 1, &value))
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain data from /proc/sys/fs/file-nr."));
+		return SYSINFO_RET_FAIL;
+	}
+
+	SET_UI64_RESULT(result, value);
+	return SYSINFO_RET_OK;
+}
