@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2015 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,11 +20,10 @@
 #ifndef ZABBIX_CHECKS_DB_H
 #define ZABBIX_CHECKS_DB_H
 
-#include "common.h"
-
 #include "dbcache.h"
-#include "sysinfo.h"
 
-int	get_value_db(DC_ITEM *item, AGENT_RESULT *result);
+#ifdef HAVE_UNIXODBC
+int	get_value_db(const DC_ITEM *item, AGENT_RESULT *result);
+#endif
 
 #endif
