@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,16 +17,15 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
 #include "checks_internal.h"
-#include "checks_java.h"
-#include "dbcache.h"
-#include "zbxself.h"
-#include "valuecache.h"
-#include "proxy.h"
-#include "preproc.h"
 
+#include "checks_java.h"
+#include "zbxself.h"
+#include "preproc.h"
+#include "zbxtrends.h"
 #include "../vmware/vmware.h"
+#include "../../libs/zbxserver/zabbix_stats.h"
+#include "../../libs/zbxsysinfo/common/zabbix_stats.h"
 
 extern unsigned char	program_type;
 
@@ -56,8 +55,6 @@ static int	compare_interfaces(const void *p1, const void *p2)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_host_interfaces_discovery                                    *
  *                                                                            *
  * Purpose: get data of all network interfaces for a host from configuration  *
  *          cache and pack into JSON for LLD                                  *
@@ -100,8 +97,7 @@ static int	zbx_host_interfaces_discovery(zbx_uint64_t hostid, struct zbx_json *j
 
 	/* pack results into JSON */
 
-	zbx_json_init(j, ZBX_JSON_STAT_BUF_LEN);
-	zbx_json_addarray(j, ZBX_PROTO_TAG_DATA);
+	zbx_json_initarray(j, ZBX_JSON_STAT_BUF_LEN);
 
 	for (i = 0; i < n; i++)
 	{
@@ -141,6 +137,23 @@ static int	zbx_host_interfaces_discovery(zbx_uint64_t hostid, struct zbx_json *j
 		{
 			zbx_snprintf(buf, sizeof(buf), "%hhu", interfaces[i].bulk);
 			zbx_json_addstring(j, "{#IF.SNMP.BULK}", buf, ZBX_JSON_TYPE_INT);
+
+			switch (interfaces[i].snmp_version)
+			{
+				case ZBX_IF_SNMP_VERSION_1:
+					p = "SNMPv1";
+					break;
+				case ZBX_IF_SNMP_VERSION_2:
+					p = "SNMPv2c";
+					break;
+				case ZBX_IF_SNMP_VERSION_3:
+					p = "SNMPv3";
+					break;
+				default:
+					p = "UNKNOWN";
+			}
+
+			zbx_json_addstring(j, "{#IF.SNMP.VERSION}", p, ZBX_JSON_TYPE_STRING);
 		}
 
 		zbx_json_close(j);
@@ -154,8 +167,6 @@ static int	zbx_host_interfaces_discovery(zbx_uint64_t hostid, struct zbx_json *j
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_value_internal                                               *
  *                                                                            *
  * Purpose: retrieve data from Zabbix server (internally supported items)     *
  *                                                                            *
@@ -164,10 +175,8 @@ static int	zbx_host_interfaces_discovery(zbx_uint64_t hostid, struct zbx_json *j
  * Return value: SUCCEED - data successfully retrieved and stored in result   *
  *               NOTSUPPORTED - requested item is not supported               *
  *                                                                            *
- * Author: Alexei Vladishev                                                   *
- *                                                                            *
  ******************************************************************************/
-int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
+int	get_value_internal(const DC_ITEM *item, AGENT_RESULT *result)
 {
 	AGENT_REQUEST	request;
 	int		ret = NOTSUPPORTED, nparams;
@@ -194,20 +203,12 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 		goto out;
 	}
 
-	if (0 == strcmp(tmp, "triggers"))			/* zabbix["triggers"] */
-	{
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			goto out;
+	if (FAIL != (ret = zbx_get_value_internal_ext(tmp, &request, result)))
+		goto out;
 
-		if (1 != nparams)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
+	ret = NOTSUPPORTED;
 
-		SET_UI64_RESULT(result, DCget_trigger_count());
-	}
-	else if (0 == strcmp(tmp, "items"))			/* zabbix["items"] */
+	if (0 == strcmp(tmp, "items"))			/* zabbix["items"] */
 	{
 		if (1 != nparams)
 		{
@@ -217,6 +218,16 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 
 		SET_UI64_RESULT(result, DCget_item_count(0));
 	}
+	else if (0 == strcmp(tmp, "version"))			/* zabbix["version"] */
+	{
+		if (1 != nparams)
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
+			goto out;
+		}
+
+		SET_STR_RESULT(result, zbx_strdup(NULL, ZABBIX_VERSION));
+	}
 	else if (0 == strcmp(tmp, "items_unsupported"))		/* zabbix["items_unsupported"] */
 	{
 		if (1 != nparams)
@@ -237,37 +248,6 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 
 		SET_UI64_RESULT(result, DCget_host_count());
 	}
-	else if (0 == strcmp(tmp, "history") ||			/* zabbix["history"] */
-			0 == strcmp(tmp, "history_log") ||	/* zabbix["history_log"] */
-			0 == strcmp(tmp, "history_str") ||	/* zabbix["history_str"] */
-			0 == strcmp(tmp, "history_text") ||	/* zabbix["history_text"] */
-			0 == strcmp(tmp, "history_uint"))	/* zabbix["history_uint"] */
-	{
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			goto out;
-
-		if (1 != nparams)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
-
-		SET_UI64_RESULT(result, DBget_row_count(tmp));
-	}
-	else if (0 == strcmp(tmp, "trends") ||			/* zabbix["trends"] */
-			0 == strcmp(tmp, "trends_uint"))	/* zabbix["trends_uint"] */
-	{
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			goto out;
-
-		if (1 != nparams)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
-
-		SET_UI64_RESULT(result, DBget_row_count(tmp));
-	}
 	else if (0 == strcmp(tmp, "queue"))			/* zabbix["queue",<from>,<to>] */
 	{
 		int	from = ZBX_QUEUE_FROM_DEFAULT, to = ZBX_QUEUE_TO_INFINITY;
@@ -342,16 +322,24 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 
 		if (0 == strcmp(tmp, "available"))		/* zabbix["host",<type>,"available"] */
 		{
+			zbx_agent_availability_t	agents[ZBX_AGENT_MAX];
+			int				i;
+
+			zbx_get_host_interfaces_availability(item->host.hostid, agents);
+
+			for (i = 0; i < ZBX_AGENT_MAX; i++)
+				zbx_free(agents[i].error);
+
 			tmp = get_rparam(&request, 1);
 
 			if (0 == strcmp(tmp, "agent"))
-				SET_UI64_RESULT(result, item->host.available);
+				SET_UI64_RESULT(result, agents[ZBX_AGENT_ZABBIX].available);
 			else if (0 == strcmp(tmp, "snmp"))
-				SET_UI64_RESULT(result, item->host.snmp_available);
+				SET_UI64_RESULT(result, agents[ZBX_AGENT_SNMP].available);
 			else if (0 == strcmp(tmp, "ipmi"))
-				SET_UI64_RESULT(result, item->host.ipmi_available);
+				SET_UI64_RESULT(result, agents[ZBX_AGENT_IPMI].available);
 			else if (0 == strcmp(tmp, "jmx"))
-				SET_UI64_RESULT(result, item->host.jmx_available);
+				SET_UI64_RESULT(result, agents[ZBX_AGENT_JMX].available);
 			else
 			{
 				SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
@@ -424,34 +412,6 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 			goto out;
 		}
 	}
-	else if (0 == strcmp(tmp, "proxy"))			/* zabbix["proxy",<hostname>,"lastaccess"] */
-	{
-		int	lastaccess;
-		char	*error = NULL;
-
-		/* this item is always processed by server */
-
-		if (3 != nparams)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
-
-		tmp = get_rparam(&request, 2);
-		if ('\0' == *tmp || 0 != strcmp(tmp, "lastaccess"))
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid third parameter."));
-			goto out;
-		}
-
-		if (FAIL == DBget_proxy_lastaccess(get_rparam(&request, 1), &lastaccess, &error))
-		{
-			SET_MSG_RESULT(result, error);
-			goto out;
-		}
-
-		SET_UI64_RESULT(result, lastaccess);
-	}
 	else if (0 == strcmp(tmp, "java"))			/* zabbix["java",...] */
 	{
 		int	res;
@@ -702,83 +662,6 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 			goto out;
 		}
 	}
-	else if (0 == strcmp(tmp, "vcache"))
-	{
-		zbx_vc_stats_t	stats;
-
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			goto out;
-
-		if (FAIL == zbx_vc_get_statistics(&stats))
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Value cache is disabled."));
-			goto out;
-		}
-
-		if (2 > nparams || nparams > 3)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
-
-		tmp = get_rparam(&request, 1);
-		if (NULL == (tmp1 = get_rparam(&request, 2)))
-			tmp1 = "";
-
-		if (0 == strcmp(tmp, "buffer"))
-		{
-			if (0 == strcmp(tmp1, "free"))
-				SET_UI64_RESULT(result, stats.free_size);
-			else if (0 == strcmp(tmp1, "pfree"))
-				SET_DBL_RESULT(result, (double)stats.free_size / stats.total_size * 100);
-			else if (0 == strcmp(tmp1, "total"))
-				SET_UI64_RESULT(result, stats.total_size);
-			else if (0 == strcmp(tmp1, "used"))
-				SET_UI64_RESULT(result, stats.total_size - stats.free_size);
-			else if (0 == strcmp(tmp1, "pused"))
-				SET_DBL_RESULT(result, (double)(stats.total_size - stats.free_size) /
-						stats.total_size * 100);
-			else
-			{
-				SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid third parameter."));
-				goto out;
-			}
-		}
-		else if (0 == strcmp(tmp, "cache"))
-		{
-			if (0 == strcmp(tmp1, "hits"))
-				SET_UI64_RESULT(result, stats.hits);
-			else if (0 == strcmp(tmp1, "requests"))
-				SET_UI64_RESULT(result, stats.hits + stats.misses);
-			else if (0 == strcmp(tmp1, "misses"))
-				SET_UI64_RESULT(result, stats.misses);
-			else if (0 == strcmp(tmp1, "mode"))
-				SET_UI64_RESULT(result, stats.mode);
-			else
-			{
-				SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid third parameter."));
-				goto out;
-			}
-		}
-		else
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
-			goto out;
-		}
-	}
-	else if (0 == strcmp(tmp, "proxy_history"))
-	{
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_PROXY))
-			goto out;
-
-		if (1 != nparams)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
-			goto out;
-		}
-
-		SET_UI64_RESULT(result, proxy_get_history_count());
-	}
 	else if (0 == strcmp(tmp, "vmware"))
 	{
 		zbx_vmware_stats_t	stats;
@@ -835,11 +718,112 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 			goto out;
 		}
 	}
+	else if (0 == strcmp(tmp, "stats"))			/* zabbix[stats,...] */
+	{
+		const char	*ip_str, *port_str, *ip;
+		unsigned short	port_number;
+		struct zbx_json	json;
+
+		if (6 < nparams)
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
+			goto out;
+		}
+
+		if (NULL == (ip_str = get_rparam(&request, 1)) || '\0' == *ip_str)
+			ip = "127.0.0.1";
+		else
+			ip = ip_str;
+
+		if (NULL == (port_str = get_rparam(&request, 2)) || '\0' == *port_str)
+		{
+			port_number = ZBX_DEFAULT_SERVER_PORT;
+		}
+		else if (SUCCEED != is_ushort(port_str, &port_number))
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid third parameter."));
+			goto out;
+		}
+
+		if (3 >= nparams)
+		{
+			if ((NULL == ip_str || '\0' == *ip_str) && (NULL == port_str || '\0' == *port_str))
+			{
+				zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
+
+				/* Adding "data" object to JSON structure to make identical JSONPath expressions */
+				/* work for both data received from internal and external source. */
+				zbx_json_addobject(&json, ZBX_PROTO_TAG_DATA);
+
+				zbx_get_zabbix_stats(&json);
+
+				zbx_json_close(&json);
+
+				set_result_type(result, ITEM_VALUE_TYPE_TEXT, json.buffer);
+
+				zbx_json_free(&json);
+			}
+			else if (SUCCEED != zbx_get_remote_zabbix_stats(ip, port_number, result))
+				goto out;
+		}
+		else
+		{
+			tmp1 = get_rparam(&request, 3);
+
+			if (0 == strcmp(tmp1, ZBX_PROTO_VALUE_ZABBIX_STATS_QUEUE))
+			{
+				tmp = get_rparam(&request, 4);		/* from */
+				tmp1 = get_rparam(&request, 5);		/* to */
+
+				if ((NULL == ip_str || '\0' == *ip_str) && (NULL == port_str || '\0' == *port_str))
+				{
+					int	from = ZBX_QUEUE_FROM_DEFAULT, to = ZBX_QUEUE_TO_INFINITY;
+
+					if (NULL != tmp && '\0' != *tmp &&
+							FAIL == is_time_suffix(tmp, &from, ZBX_LENGTH_UNLIMITED))
+					{
+						SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid fifth parameter."));
+						goto out;
+					}
+
+					if (NULL != tmp1 && '\0' != *tmp1 &&
+							FAIL == is_time_suffix(tmp1, &to, ZBX_LENGTH_UNLIMITED))
+					{
+						SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid sixth parameter."));
+						goto out;
+					}
+
+					if (ZBX_QUEUE_TO_INFINITY != to && from > to)
+					{
+						SET_MSG_RESULT(result, zbx_strdup(NULL, "Parameters represent an"
+								" invalid interval."));
+						goto out;
+					}
+
+					zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
+
+					zbx_json_adduint64(&json, ZBX_PROTO_VALUE_ZABBIX_STATS_QUEUE,
+							DCget_item_queue(NULL, from, to));
+
+					set_result_type(result, ITEM_VALUE_TYPE_TEXT, json.buffer);
+
+					zbx_json_free(&json);
+				}
+				else if (SUCCEED != zbx_get_remote_zabbix_stats_queue(ip, port_number, tmp, tmp1,
+						result))
+				{
+					goto out;
+				}
+			}
+			else
+			{
+				SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid forth parameter."));
+				goto out;
+			}
+		}
+	}
 	else if (0 == strcmp(tmp, "preprocessing_queue"))
 	{
-		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			goto out;
-
 		if (1 != nparams)
 		{
 			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
@@ -848,6 +832,83 @@ int	get_value_internal(DC_ITEM *item, AGENT_RESULT *result)
 
 		SET_UI64_RESULT(result, zbx_preprocessor_get_queue_size());
 	}
+	else if (0 == strcmp(tmp, "tcache"))			/* zabbix[tcache,cache,<parameter>] */
+	{
+		char		*error = NULL;
+		zbx_tfc_stats_t	stats;
+
+		if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid first parameter."));
+			goto out;
+		}
+
+		if (2 > nparams || 3 < nparams)
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
+			goto out;
+		}
+
+		tmp1 = get_rparam(&request, 1);
+
+		if (0 != strcmp(tmp1, "cache"))
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
+			goto out;
+		}
+
+		tmp = get_rparam(&request, 2);
+
+		if (FAIL == zbx_tfc_get_stats(&stats, &error))
+		{
+			SET_MSG_RESULT(result, error);
+			goto out;
+		}
+
+		if (NULL == tmp || 0 == strcmp(tmp, "all"))
+		{
+			SET_UI64_RESULT(result, stats.hits + stats.misses);
+		}
+		else if (0 == strcmp(tmp, "hits"))
+		{
+			SET_UI64_RESULT(result, stats.hits);
+		}
+		else if (0 == strcmp(tmp, "misses"))
+		{
+			SET_UI64_RESULT(result, stats.misses);
+		}
+		else if (0 == strcmp(tmp, "items"))
+		{
+			SET_UI64_RESULT(result, stats.items_num);
+		}
+		else if (0 == strcmp(tmp, "requests"))
+		{
+			SET_UI64_RESULT(result, stats.requests_num);
+		}
+		else if (0 == strcmp(tmp, "pmisses"))
+		{
+			zbx_uint64_t	total = stats.hits + stats.misses;
+
+			SET_DBL_RESULT(result, (0 == total ? 0 : (double)stats.misses / total * 100));
+		}
+		else if (0 == strcmp(tmp, "phits"))
+		{
+			zbx_uint64_t	total = stats.hits + stats.misses;
+
+			SET_DBL_RESULT(result, (0 == total ? 0 : (double)stats.hits / total * 100));
+		}
+		else if (0 == strcmp(tmp, "pitems"))
+		{
+			zbx_uint64_t	total = stats.items_num + stats.requests_num;
+
+			SET_DBL_RESULT(result, (0 == total ? 0 : (double)stats.items_num / total * 100));
+		}
+		else
+		{
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid third parameter."));
+			goto out;
+		}
+	}
 	else
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid first parameter."));
