@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,13 +17,12 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "dbcache.h"
+#include "snmptrapper.h"
+
 #include "zbxself.h"
 #include "daemon.h"
 #include "log.h"
 #include "proxy.h"
-#include "snmptrapper.h"
 #include "zbxserver.h"
 #include "zbxregexp.h"
 #include "preproc.h"
@@ -35,8 +34,9 @@ static char	*buffer = NULL;
 static int	offset = 0;
 static int	force = 0;
 
-extern unsigned char	process_type, program_type;
-extern int		server_num, process_num;
+extern ZBX_THREAD_LOCAL unsigned char	process_type;
+extern unsigned char			program_type;
+extern ZBX_THREAD_LOCAL int		server_num, process_num;
 
 static void	DBget_lastsize(void)
 {
@@ -68,16 +68,12 @@ static void	DBupdate_lastsize(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: process_trap_for_interface                                       *
  *                                                                            *
  * Purpose: add trap to all matching items for the specified interface        *
  *                                                                            *
  * Return value: SUCCEED - a matching item was found                          *
  *               FAIL - no matching item was found (including fallback items) *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static int	process_trap_for_interface(zbx_uint64_t interfaceid, char *trap, zbx_timespec_t *ts)
 {
@@ -87,7 +83,6 @@ static int	process_trap_for_interface(zbx_uint64_t interfaceid, char *trap, zbx_
 	size_t			num, i;
 	int			ret = FAIL, fb = -1, *lastclocks = NULL, *errcodes = NULL, value_type, regexp_ret;
 	zbx_uint64_t		*itemids = NULL;
-	unsigned char		*states = NULL;
 	AGENT_RESULT		*results = NULL;
 	AGENT_REQUEST		request;
 	zbx_vector_ptr_t	regexps;
@@ -97,7 +92,6 @@ static int	process_trap_for_interface(zbx_uint64_t interfaceid, char *trap, zbx_
 	num = DCconfig_get_snmp_items_by_interfaceid(interfaceid, &items);
 
 	itemids = (zbx_uint64_t *)zbx_malloc(itemids, sizeof(zbx_uint64_t) * num);
-	states = (unsigned char *)zbx_malloc(states, sizeof(unsigned char) * num);
 	lastclocks = (int *)zbx_malloc(lastclocks, sizeof(int) * num);
 	errcodes = (int *)zbx_malloc(errcodes, sizeof(int) * num);
 	results = (AGENT_RESULT *)zbx_malloc(results, sizeof(AGENT_RESULT) * num);
@@ -108,7 +102,7 @@ static int	process_trap_for_interface(zbx_uint64_t interfaceid, char *trap, zbx_
 		errcodes[i] = FAIL;
 
 		items[i].key = zbx_strdup(items[i].key, items[i].key_orig);
-		if (SUCCEED != substitute_key_macros(&items[i].key, NULL, &items[i], NULL,
+		if (SUCCEED != substitute_key_macros(&items[i].key, NULL, &items[i], NULL, NULL,
 				MACRO_TYPE_ITEM_KEY, error, sizeof(error)))
 		{
 			SET_MSG_RESULT(&results[i], zbx_strdup(NULL, error));
@@ -190,20 +184,18 @@ next:
 				}
 
 				items[i].state = ITEM_STATE_NORMAL;
-				zbx_preprocess_item_value(items[i].itemid, items[i].value_type, items[i].flags,
+				zbx_preprocess_item_value(items[i].itemid, items[i].host.hostid, items[i].value_type, items[i].flags,
 						&results[i], ts, items[i].state, NULL);
 
 				itemids[i] = items[i].itemid;
-				states[i] = items[i].state;
 				lastclocks[i] = ts->sec;
 				break;
 			case NOTSUPPORTED:
 				items[i].state = ITEM_STATE_NOTSUPPORTED;
-				zbx_preprocess_item_value(items[i].itemid, items[i].value_type, items[i].flags, NULL,
+				zbx_preprocess_item_value(items[i].itemid, items[i].host.hostid, items[i].value_type, items[i].flags, NULL,
 						ts, items[i].state, results[i].msg);
 
 				itemids[i] = items[i].itemid;
-				states[i] = items[i].state;
 				lastclocks[i] = ts->sec;
 				break;
 		}
@@ -214,11 +206,10 @@ next:
 
 	zbx_free(results);
 
-	DCrequeue_items(itemids, states, lastclocks, errcodes, num);
+	DCrequeue_items(itemids, lastclocks, errcodes, num);
 
 	zbx_free(errcodes);
 	zbx_free(lastclocks);
-	zbx_free(states);
 	zbx_free(itemids);
 
 	DCconfig_clean_items(items, NULL, num);
@@ -233,8 +224,6 @@ next:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: process_trap                                                     *
  *                                                                            *
  * Purpose: process a single trap                                             *
  *                                                                            *
@@ -242,8 +231,6 @@ next:
  *             begin - [IN] beginning of the trap message                     *
  *             end - [IN] end of the trap message                             *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static void	process_trap(const char *addr, char *begin, char *end)
 {
@@ -280,13 +267,9 @@ static void	process_trap(const char *addr, char *begin, char *end)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: parse_traps                                                      *
  *                                                                            *
  * Purpose: split traps and process them with process_trap()                  *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static void	parse_traps(int flag)
 {
@@ -390,8 +373,6 @@ static void	parse_traps(int flag)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: delay_trap_logs                                                  *
  *                                                                            *
  * Purpose: delay SNMP trapper file related issue log entries for 60 seconds  *
  *          unless this is the first time this issue has occurred             *
@@ -419,21 +400,16 @@ static void	delay_trap_logs(char *error, int log_level)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: read_traps                                                       *
  *                                                                            *
  * Purpose: read the traps and then parse them with parse_traps()             *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static int	read_traps(void)
 {
-	const char	*__function_name = "read_traps";
-	int		nbytes = 0;
-	char		*error = NULL;
+	int	nbytes = 0;
+	char	*error = NULL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() lastsize: %lld", __function_name, (long long int)trap_lastsize);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() lastsize: %lld", __func__, (long long int)trap_lastsize);
 
 	if (-1 == lseek(trap_fd, trap_lastsize, SEEK_SET))
 	{
@@ -461,19 +437,15 @@ static int	read_traps(void)
 out:
 	zbx_free(error);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
 	return nbytes;
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: close_trap_file                                                  *
  *                                                                            *
  * Purpose: close trap file and reset lastsize                                *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  * Comments: !!! do not reset lastsize elsewhere !!!                          *
  *                                                                            *
  ******************************************************************************/
@@ -488,29 +460,17 @@ static void	close_trap_file(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: open_trap_file                                                   *
  *                                                                            *
  * Purpose: open the trap file and get it's node number                       *
  *                                                                            *
  * Return value: file descriptor of the opened file or -1 otherwise           *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static int	open_trap_file(void)
 {
 	zbx_stat_t	file_buf;
 	char		*error = NULL;
 
-	if (0 != zbx_stat(CONFIG_SNMPTRAP_FILE, &file_buf))
-	{
-		error = zbx_dsprintf(error, "cannot stat SNMP trapper file \"%s\": %s", CONFIG_SNMPTRAP_FILE,
-				zbx_strerror(errno));
-		delay_trap_logs(error, LOG_LEVEL_CRIT);
-		goto out;
-	}
-
 	if (-1 == (trap_fd = open(CONFIG_SNMPTRAP_FILE, O_RDONLY)))
 	{
 		if (ENOENT != errno)	/* file exists but cannot be opened */
@@ -522,6 +482,16 @@ static int	open_trap_file(void)
 		goto out;
 	}
 
+	if (0 != zbx_fstat(trap_fd, &file_buf))
+	{
+		error = zbx_dsprintf(error, "cannot stat SNMP trapper file \"%s\": %s", CONFIG_SNMPTRAP_FILE,
+				zbx_strerror(errno));
+		delay_trap_logs(error, LOG_LEVEL_CRIT);
+		close(trap_fd);
+		trap_fd = -1;
+		goto out;
+	}
+
 	trap_ino = file_buf.st_ino;	/* a new file was opened */
 out:
 	zbx_free(error);
@@ -530,8 +500,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_latest_data                                                  *
  *                                                                            *
  * Purpose: Open the latest trap file. If the current file has been rotated,  *
  *          process that and then open the latest file.                       *
@@ -539,8 +507,6 @@ out:
  * Return value: SUCCEED - there are new traps to be parsed                   *
  *               FAIL - there are no new traps or trap file does not exist    *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 static int	get_latest_data(void)
 {
@@ -609,18 +575,13 @@ static int	get_latest_data(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: main_snmptrapper_loop                                            *
  *                                                                            *
  * Purpose: SNMP trap reader's entry point                                    *
  *                                                                            *
- * Author: Rudolfs Kreicbergs                                                 *
- *                                                                            *
  ******************************************************************************/
 ZBX_THREAD_ENTRY(snmptrapper_thread, args)
 {
-	const char	*__function_name = "main_snmptrapper_loop";
-	double		sec;
+	double	sec;
 
 	process_type = ((zbx_thread_args_t *)args)->process_type;
 	server_num = ((zbx_thread_args_t *)args)->server_num;
@@ -629,7 +590,9 @@ ZBX_THREAD_ENTRY(snmptrapper_thread, args)
 	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() trapfile:'%s'", __function_name, CONFIG_SNMPTRAP_FILE);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() trapfile:'%s'", __func__, CONFIG_SNMPTRAP_FILE);
+
+	update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);
 
 	zbx_setproctitle("%s [connecting to the database]", get_process_type_string(process_type));
 
@@ -640,14 +603,14 @@ ZBX_THREAD_ENTRY(snmptrapper_thread, args)
 	buffer = (char *)zbx_malloc(buffer, MAX_BUFFER_LEN);
 	*buffer = '\0';
 
-	for (;;)
+	while (ZBX_IS_RUNNING())
 	{
 		sec = zbx_time();
-		zbx_update_env(sec);
+		zbx_update_env(get_process_type_string(process_type), sec);
 
 		zbx_setproctitle("%s [processing data]", get_process_type_string(process_type));
 
-		while (SUCCEED == get_latest_data())
+		while (ZBX_IS_RUNNING() && SUCCEED == get_latest_data())
 			read_traps();
 		sec = zbx_time() - sec;
 
@@ -661,4 +624,9 @@ ZBX_THREAD_ENTRY(snmptrapper_thread, args)
 
 	if (-1 != trap_fd)
 		close(trap_fd);
+
+	zbx_setproctitle("%s #%d [terminated]", get_process_type_string(process_type), process_num);
+
+	while (1)
+		zbx_sleep(SEC_PER_MIN);
 }
